@@ -25,6 +25,12 @@
  */
 
 #include <config.h>
+#include <stdio.h>
+#include <unicode.h>
+#include <ctype.h>
+#include <math.h>
+#include <string.h>
+#include <gdk/gdkx.h> /* for BlackPixel */
 #include <gtk/gtkenums.h>
 #include <gtk/gtkentry.h>
 #include <gtk/gtkwindow.h>
@@ -32,22 +38,18 @@
 #include <gtk/gtksignal.h>
 #include <gdk/gdkkeysyms.h>
 #include <libgnomeui/gnome-canvas.h>
-#include <stdio.h>
-#include <unicode.h>
+#include <libgnomeui/gnome-canvas-rect-ellipse.h>
 #include "e-cell-text.h"
 #include "gal/util/e-util.h"
 #include "gal/widgets/e-canvas.h"
 #include "gal/widgets/e-font.h"
 #include "gal/widgets/e-unicode.h"
 #include "e-table-item.h"
-#include "gal/e-text/e-text-event-processor.h"
+#include "gal/util/e-text-event-processor.h"
 #include "gal/e-text/e-text.h"
-#include "gal/e-text/e-text-event-processor-emacs-like.h"
+#include "gal/util/e-text-event-processor-emacs-like.h"
 #include "e-table-tooltip.h"
 
-#include <gdk/gdkx.h> /* for BlackPixel */
-#include <ctype.h>
-#include <math.h>
 
 #define ECT_CLASS(c) (E_CELL_TEXT_CLASS(GTK_OBJECT((c))->klass))
 
@@ -66,6 +68,7 @@ enum {
 	ARG_STRIKEOUT_COLUMN,
 	ARG_BOLD_COLUMN,
 	ARG_COLOR_COLUMN,
+	ARG_EDITABLE,
 };
 
 
@@ -93,11 +96,7 @@ typedef struct _CellEdit CellEdit;
 typedef struct {
 	ECellView    cell_view;
 	GdkGC       *gc;
-#if 0
-	GdkFont     *font;
-#else
 	EFont *font;
-#endif
 	GdkCursor *i_cursor;
 	GdkBitmap *stipple;		/* Stipple for text */
 	
@@ -322,27 +321,12 @@ ect_cancel_edit (ECellTextView *text_view)
 static ECellView *
 ect_new_view (ECell *ecell, ETableModel *table_model, void *e_table_item_view)
 {
-	ECellText *ect = E_CELL_TEXT (ecell);
 	ECellTextView *text_view = g_new0 (ECellTextView, 1);
 	GnomeCanvas *canvas = GNOME_CANVAS_ITEM (e_table_item_view)->canvas;
 	
 	text_view->cell_view.ecell = ecell;
 	text_view->cell_view.e_table_model = table_model;
 	text_view->cell_view.e_table_item_view = e_table_item_view;
-	
-	if (ect->font_name){
-#if 0
-		GdkFont *f;
-
-		f = gdk_fontset_load (ect->font_name);
-		text_view->font = f;
-#endif
-		text_view->font = e_font_from_gdk_name (ect->font_name);
-	}
-	if (!text_view->font){
-		gdk_font_ref (GTK_WIDGET (canvas)->style->font);
-		text_view->font = e_font_from_gdk_font (GTK_WIDGET (canvas)->style->font);
-	}
 
 	text_view->canvas = canvas;
 
@@ -370,11 +354,20 @@ static void
 ect_realize (ECellView *ecell_view)
 {
 	ECellTextView *text_view = (ECellTextView *) ecell_view;
+	ECellText *ect = (ECellText *) ecell_view->ecell;
 	
 	text_view->gc = gdk_gc_new (GTK_WIDGET (text_view->canvas)->window);
 
 	text_view->i_cursor = gdk_cursor_new (GDK_XTERM);
 	
+	if (ect->font_name){
+		text_view->font = e_font_from_gdk_name (ect->font_name);
+	}
+	if (!text_view->font){
+		gdk_font_ref (GTK_WIDGET (text_view->canvas)->style->font);
+		text_view->font = e_font_from_gdk_font (GTK_WIDGET (text_view->canvas)->style->font);
+	}
+	
 	calc_ellipsis (text_view);
 
 	if (parent_class->realize)
@@ -586,6 +579,7 @@ ect_draw (ECellView *ecell_view, GdkDrawable *drawable,
 				sel_rect.height = height;
 				gtk_paint_flat_box (canvas->style,
 						    drawable,
+
 						    edit->has_selection ? GTK_STATE_SELECTED : GTK_STATE_ACTIVE,
 						    GTK_SHADOW_NONE,
 						    clip_rect,
@@ -803,8 +797,6 @@ ect_event (ECellView *ecell_view, GdkEvent *event, int model_col, int view_col,
 			}
 		}
 
-		else
-			return 0;
 		break;
 	case GDK_BUTTON_PRESS: /* Fall Through */
 	case GDK_BUTTON_RELEASE:
@@ -1112,37 +1104,34 @@ tooltip_event (GtkWidget *window,
 	
 	switch (event->type) {
 	case GDK_LEAVE_NOTIFY:
-		if (tooltip->window) {
-			gtk_widget_destroy (tooltip->window);
-			tooltip->window = NULL;
-		}
+		e_canvas_hide_tooltip (E_CANVAS(GNOME_CANVAS_ITEM(tooltip->eti)->canvas));
 		break;
 	case GDK_BUTTON_PRESS:
 	case GDK_BUTTON_RELEASE:
 		if (event->type == GDK_BUTTON_RELEASE) {
-			if (tooltip->window) {
-				gtk_widget_destroy (tooltip->window);
-				tooltip->window = NULL;
-			}
+			e_canvas_hide_tooltip (E_CANVAS(GNOME_CANVAS_ITEM(tooltip->eti)->canvas));
 		}
 
 		event->button.x = tooltip->cx;
 		event->button.y = tooltip->cy;
 		gtk_signal_emit_by_name (GTK_OBJECT (tooltip->eti), "event",
 					 event, &ret_val);
+		if (!ret_val)
+			gtk_propagate_event (GTK_WIDGET(GNOME_CANVAS_ITEM(tooltip->eti)->canvas), event);
+		ret_val = TRUE;
 		break;
 	case GDK_KEY_PRESS:
-		if (tooltip->window){
-			gtk_widget_destroy (tooltip->window);
-			tooltip->window = NULL;
-		}
+		e_canvas_hide_tooltip (E_CANVAS(GNOME_CANVAS_ITEM(tooltip->eti)->canvas));
 		gtk_signal_emit_by_name (GTK_OBJECT (tooltip->eti), "event",
 					 event, &ret_val);
+		if (!ret_val)
+			gtk_propagate_event (GTK_WIDGET(GNOME_CANVAS_ITEM(tooltip->eti)->canvas), event);
+		ret_val = TRUE;
 		break;
 	default:
 		break;
 	}
-	
+
 	return ret_val;
 }
 
@@ -1172,6 +1161,8 @@ ect_show_tooltip (ECellView *ecell_view,
 	double tooltip_y;
 	GnomeCanvasItem *rect;
 	double text_height;
+	ECellText *ect = E_CELL_TEXT(ecell_view->ecell);
+	GtkWidget *window;
 
 	tooltip->timer = 0;
 
@@ -1209,11 +1200,11 @@ ect_show_tooltip (ECellView *ecell_view,
 	pixel_origin.x -= (int) gtk_layout_get_hadjustment (GTK_LAYOUT (text_view->canvas))->value;
 	pixel_origin.y -= (int) gtk_layout_get_vadjustment (GTK_LAYOUT (text_view->canvas))->value;
 
-	tooltip->window = gtk_window_new (GTK_WINDOW_POPUP);
-	gtk_container_set_border_width (GTK_CONTAINER (tooltip->window), 1);
+	window = gtk_window_new (GTK_WINDOW_POPUP);
+	gtk_container_set_border_width (GTK_CONTAINER (window), 1);
 
 	canvas = e_canvas_new ();
-	gtk_container_add (GTK_CONTAINER (tooltip->window), canvas);
+	gtk_container_add (GTK_CONTAINER (window), canvas);
 
 	max_width = 0.0;
 	for (lines = cell.breaks->lines, i = 0; i < cell.breaks->num_lines;
@@ -1233,13 +1224,16 @@ ect_show_tooltip (ECellView *ecell_view,
 				      "y1", (double) 0.0,
 				      "x2", (double) max_width + 4,
 				      "y2", (double) text_height,
-				      "fill_color", "light gray",
+				      "fill_color_gdk", tooltip->background,
 				      NULL);
 
 	tooltip_text = gnome_canvas_item_new (gnome_canvas_root (GNOME_CANVAS (canvas)),
 					      e_text_get_type (),
 					      "anchor", GTK_ANCHOR_NW,
 /*  					      "font_gdk", text_view->font, */
+					      "bold", (gboolean) ect->bold_column >= 0 && e_table_model_value_at(ecell_view->e_table_model, ect->bold_column, row),
+					      "strikeout", (gboolean) ect->strikeout_column >= 0 && e_table_model_value_at(ecell_view->e_table_model, ect->strikeout_column, row),
+					      "fill_color_gdk", tooltip->foreground,
 					      "text", cell.text,
 					      "editable", FALSE,
 					      "clip_width", max_width,
@@ -1272,18 +1266,18 @@ ect_show_tooltip (ECellView *ecell_view,
 			       "x2", (double) tooltip_width + 6,
 			       "y2", (double) tooltip->row_height + 1,
 			       NULL);
-	gtk_widget_set_usize (tooltip->window, tooltip_width + 6,
+	gtk_widget_set_usize (window, tooltip_width + 6,
 			      tooltip->row_height + 1);
 	gnome_canvas_set_scroll_region (GNOME_CANVAS (canvas), 0.0, 0.0,
 					(double) tooltip_width + 6,
 					(double) tooltip_height);
 	gtk_widget_show (canvas);
-	gtk_widget_realize (tooltip->window);
-	gtk_signal_connect (GTK_OBJECT (tooltip->window), "event",
+	gtk_widget_realize (window);
+	gtk_signal_connect (GTK_OBJECT (window), "event",
 			    GTK_SIGNAL_FUNC (tooltip_event), tooltip);
 
-	gtk_widget_popup (tooltip->window, pixel_origin.x + tooltip->x,
-			  pixel_origin.y + tooltip->y + 1);
+	e_canvas_popup_tooltip (E_CANVAS(text_view->canvas), window, pixel_origin.x + tooltip->x,
+				pixel_origin.y + tooltip->y - 1);
 
 	unref_lines (&cell);
 	unbuild_current_cell (&cell);
@@ -1324,6 +1318,10 @@ ect_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 		text->color_column = GTK_VALUE_INT (*arg);
 		break;
 
+	case ARG_EDITABLE:
+		text->editable = GTK_VALUE_BOOL (*arg) ? TRUE : FALSE;
+		break;
+
 	default:
 		return;
 	}
@@ -1350,6 +1348,10 @@ ect_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 		GTK_VALUE_INT (*arg) = text->color_column;
 		break;
 
+	case ARG_EDITABLE:
+		GTK_VALUE_BOOL (*arg) = text->editable ? TRUE : FALSE;
+		break;
+
 	default:
 		arg->type = GTK_TYPE_INVALID;
 		break;
@@ -1392,6 +1394,8 @@ e_cell_text_class_init (GtkObjectClass *object_class)
 				 GTK_TYPE_INT, GTK_ARG_READWRITE, ARG_BOLD_COLUMN);
 	gtk_object_add_arg_type ("ECellText::color_column",
 				 GTK_TYPE_INT, GTK_ARG_READWRITE, ARG_COLOR_COLUMN);
+	gtk_object_add_arg_type ("ECellText::editable",
+				 GTK_TYPE_BOOL, GTK_ARG_READWRITE, ARG_EDITABLE);
 
 	if (!clipboard_atom)
 		clipboard_atom = gdk_atom_intern ("CLIPBOARD", FALSE);
@@ -1405,6 +1409,7 @@ e_cell_text_init (ECellText *ect)
 	ect->strikeout_column = -1;
 	ect->bold_column = -1;
 	ect->color_column = -1;
+	ect->editable = TRUE;
 }
 
 E_MAKE_TYPE(e_cell_text, "ECellText", ECellText, e_cell_text_class_init, e_cell_text_init, PARENT_TYPE);
@@ -1888,6 +1893,7 @@ e_cell_text_view_command (ETextEventProcessor *tep, ETextEventProcessorCommand *
 	CellEdit *edit = (CellEdit *) data;
 	CurrentCell *cell = CURRENT_CELL(edit);
 	ECellTextView *text_view = cell->text_view;
+	ECellText *ect = E_CELL_TEXT (text_view->cell_view.ecell);
 
 	gboolean change = FALSE;
 	gboolean redraw = FALSE;
@@ -1897,6 +1903,14 @@ e_cell_text_view_command (ETextEventProcessor *tep, ETextEventProcessorCommand *
 	
 	font = text_view->font;
 
+	/* If the EText isn't editable, then ignore any commands that would
+	   modify the text. */
+	if (!ect->editable && (command->action == E_TEP_DELETE
+			       || command->action == E_TEP_INSERT
+			       || command->action == E_TEP_PASTE
+			       || command->action == E_TEP_GET_SELECTION))
+		return;
+
 	switch (command->action) {
 	case E_TEP_MOVE:
 		edit->selection_start = _get_position (text_view, command);
