@@ -10,6 +10,7 @@
  */
 
 #include <config.h>
+#include <gdk/gdkkeysyms.h>
 #include <gtk/gtksignal.h>
 #include <libgnomeui/gnome-canvas-rect-ellipse.h>
 #include "e-table-group-container.h"
@@ -34,10 +35,11 @@ enum {
 	ARG_WIDTH,
 	ARG_MINIMUM_WIDTH,
 	ARG_FROZEN,
-	ARG_TABLE_DRAW_GRID,
+	ARG_TABLE_HORIZONTAL_DRAW_GRID,
+	ARG_TABLE_VERTICAL_DRAW_GRID,
 	ARG_TABLE_DRAW_FOCUS,
 	ARG_CURSOR_MODE,
-	ARG_TABLE_SELECTION_MODEL,
+	ARG_SELECTION_MODEL,
 	ARG_LENGTH_THRESHOLD,
 };
 
@@ -98,8 +100,8 @@ etgc_destroy (GtkObject *object)
 	if (etgc->sort_info)
 		gtk_object_unref (GTK_OBJECT(etgc->sort_info));
 
-	if (etgc->table_selection_model)
-		gtk_object_unref (GTK_OBJECT(etgc->table_selection_model));
+	if (etgc->selection_model)
+		gtk_object_unref (GTK_OBJECT(etgc->selection_model));
 
 	if (etgc->rect)
 		gtk_object_destroy (GTK_OBJECT(etgc->rect));
@@ -387,6 +389,58 @@ child_key_press (ETableGroup *etg, int row, int col, GdkEvent *event,
 	return e_table_group_key_press (E_TABLE_GROUP (etgc), row, col, event);
 }
 
+static ETableGroupContainerChildNode *
+create_child_node (ETableGroupContainer *etgc, void *val)
+{
+	ETableGroup *child;
+	ETableGroupContainerChildNode *child_node;
+	ETableGroup *etg = E_TABLE_GROUP(etgc);
+
+	child_node = g_new (ETableGroupContainerChildNode, 1);
+	child_node->rect = gnome_canvas_item_new (GNOME_CANVAS_GROUP (etgc),
+						  gnome_canvas_rect_get_type (),
+						  "fill_color", "grey70",
+						  "outline_color", "grey50",
+						  NULL);
+	child_node->text = gnome_canvas_item_new (GNOME_CANVAS_GROUP (etgc),
+						  e_text_get_type (),
+						  "font_gdk", etgc->font,
+						  "anchor", GTK_ANCHOR_SW,
+						  "fill_color", "black",
+						  "draw_background", FALSE,
+						  NULL);
+	child = e_table_group_new (GNOME_CANVAS_GROUP (etgc), etg->full_header,
+				   etg->header, etg->model, etgc->sort_info, etgc->n + 1);
+	gnome_canvas_item_set(GNOME_CANVAS_ITEM(child),
+			      "horizontal_draw_grid", etgc->horizontal_draw_grid,
+			      "vertical_draw_grid", etgc->vertical_draw_grid,
+			      "drawfocus", etgc->draw_focus,
+			      "cursor_mode", etgc->cursor_mode,
+			      "selection_model", etgc->selection_model,
+			      "length_threshold", etgc->length_threshold,
+			      "minimum_width", etgc->minimum_width - GROUP_INDENT,
+			      NULL);
+
+	gtk_signal_connect (GTK_OBJECT (child), "cursor_change",
+			    GTK_SIGNAL_FUNC (child_cursor_change), etgc);
+	gtk_signal_connect (GTK_OBJECT (child), "cursor_activated",
+			    GTK_SIGNAL_FUNC (child_cursor_activated), etgc);
+	gtk_signal_connect (GTK_OBJECT (child), "double_click",
+			    GTK_SIGNAL_FUNC (child_double_click), etgc);
+	gtk_signal_connect (GTK_OBJECT (child), "right_click",
+			    GTK_SIGNAL_FUNC (child_right_click), etgc);
+	gtk_signal_connect (GTK_OBJECT (child), "click",
+			    GTK_SIGNAL_FUNC (child_click), etgc);
+	gtk_signal_connect (GTK_OBJECT (child), "key_press",
+			    GTK_SIGNAL_FUNC (child_key_press), etgc);
+	child_node->child = child;
+	child_node->key = e_table_model_duplicate_value (etg->model, etgc->ecol->col_idx, val);
+	child_node->string = e_table_model_value_to_string (etg->model, etgc->ecol->col_idx, val);
+	child_node->count = 0;
+
+	return child_node;
+}
+
 static void
 etgc_add (ETableGroup *etg, gint row)
 {
@@ -414,45 +468,8 @@ etgc_add (ETableGroup *etg, gint row)
 		    (comp_val < 0 && (!etgc->ascending)))
 			break;
 	}
-	child_node = g_new (ETableGroupContainerChildNode, 1);
-	child_node->rect = gnome_canvas_item_new (GNOME_CANVAS_GROUP (etgc),
-						  gnome_canvas_rect_get_type (),
-						  "fill_color", "grey70",
-						  "outline_color", "grey50",
-						  NULL);
-	child_node->text = gnome_canvas_item_new (GNOME_CANVAS_GROUP (etgc),
-						  e_text_get_type (),
-						  "font_gdk", etgc->font,
-						  "anchor", GTK_ANCHOR_SW,
-						  "fill_color", "black",
-						  "draw_background", FALSE,
-						  NULL);
-	child = e_table_group_new (GNOME_CANVAS_GROUP (etgc), etg->full_header,
-				   etg->header, etg->model, etgc->sort_info, etgc->n + 1);
-	gnome_canvas_item_set(GNOME_CANVAS_ITEM(child),
-			      "drawgrid", etgc->draw_grid,
-			      "drawfocus", etgc->draw_focus,
-			      "cursor_mode", etgc->cursor_mode,
-			      "table_selection_model", etgc->table_selection_model,
-			      "length_threshold", etgc->length_threshold,
-			      "minimum_width", etgc->minimum_width - GROUP_INDENT,
-			      NULL);
-
-	gtk_signal_connect (GTK_OBJECT (child), "cursor_change",
-			    GTK_SIGNAL_FUNC (child_cursor_change), etgc);
-	gtk_signal_connect (GTK_OBJECT (child), "cursor_activated",
-			    GTK_SIGNAL_FUNC (child_cursor_activated), etgc);
-	gtk_signal_connect (GTK_OBJECT (child), "double_click",
-			    GTK_SIGNAL_FUNC (child_double_click), etgc);
-	gtk_signal_connect (GTK_OBJECT (child), "right_click",
-			    GTK_SIGNAL_FUNC (child_right_click), etgc);
-	gtk_signal_connect (GTK_OBJECT (child), "click",
-			    GTK_SIGNAL_FUNC (child_click), etgc);
-	gtk_signal_connect (GTK_OBJECT (child), "key_press",
-			    GTK_SIGNAL_FUNC (child_key_press), etgc);
-	child_node->child = child;
-	child_node->key = e_table_model_duplicate_value (etg->model, etgc->ecol->col_idx, val);
-	child_node->string = e_table_model_value_to_string (etg->model, etgc->ecol->col_idx, val);
+	child_node = create_child_node (etgc, val);
+	child = child_node->child;
 	child_node->count = 1;
 	e_table_group_add (child, row);
 
@@ -465,13 +482,67 @@ etgc_add (ETableGroup *etg, gint row)
 	e_canvas_item_request_reflow (GNOME_CANVAS_ITEM (etgc));
 }
 
+static void
+etgc_add_array (ETableGroup *etg, const int *array, int count)
+{
+	int i;
+	ETableGroupContainer *etgc = E_TABLE_GROUP_CONTAINER (etg);
+	void *lastval = 0;
+	int laststart = 0;
+	GCompareFunc comp = etgc->ecol->compare;
+	ETableGroupContainerChildNode *child_node;
+	ETableGroup *child;
+
+	if (count <= 0)
+		return;
+
+	e_table_group_container_list_free (etgc);
+	etgc->children = NULL;
+
+	lastval = e_table_model_value_at (etg->model, etgc->ecol->col_idx, array[0]);
+
+	for (i = 1; i < count; i++) {
+		void *val = e_table_model_value_at (etg->model, etgc->ecol->col_idx, array[i]);
+		int comp_val;
+
+		comp_val = (*comp)(lastval, val);
+		if (comp_val != 0) {
+			child_node = create_child_node(etgc, lastval);
+			child = child_node->child;
+
+			e_table_group_add_array(child, array + laststart, i - laststart);
+			child_node->count = i - laststart;
+
+			etgc->children = g_list_append (etgc->children, child_node);
+			compute_text (etgc, child_node);
+			laststart = i;
+			lastval = val;
+		}
+	}
+
+	child_node = create_child_node(etgc, lastval);
+	child = child_node->child;
+
+	e_table_group_add_array(child, array + laststart, i - laststart);
+	child_node->count = i - laststart;
+
+	etgc->children = g_list_append (etgc->children, child_node);
+	compute_text (etgc, child_node);
+
+	e_canvas_item_request_reflow (GNOME_CANVAS_ITEM (etgc));
+}
+
 static void
 etgc_add_all (ETableGroup *etg)
 {
-	int rows = e_table_model_row_count(etg->model);
-	int i;
-	for (i = 0; i < rows; i++)
-		etgc_add(etg, i);
+	ETableGroupContainer *etgc = E_TABLE_GROUP_CONTAINER (etg);
+	ESorter *sorter = etgc->selection_model->sorter;
+	int *array;
+	int count;
+
+	e_sorter_get_sorted_to_model_array(sorter, &array, &count);
+
+	etgc_add_array(etg, array, count);
 }
 
 static gboolean
@@ -636,26 +707,36 @@ etgc_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 		}
 		break;
 
-	case ARG_TABLE_SELECTION_MODEL:
-		if (etgc->table_selection_model)
-			gtk_object_unref(GTK_OBJECT(etgc->table_selection_model));
-		etgc->table_selection_model = E_TABLE_SELECTION_MODEL(GTK_VALUE_OBJECT (*arg));
-		if (etgc->table_selection_model)
-			gtk_object_ref(GTK_OBJECT(etgc->table_selection_model));
+	case ARG_SELECTION_MODEL:
+		if (etgc->selection_model)
+			gtk_object_unref(GTK_OBJECT(etgc->selection_model));
+		etgc->selection_model = E_SELECTION_MODEL(GTK_VALUE_OBJECT (*arg));
+		if (etgc->selection_model)
+			gtk_object_ref(GTK_OBJECT(etgc->selection_model));
 		for (list = etgc->children; list; list = g_list_next (list)) {
 			ETableGroupContainerChildNode *child_node = (ETableGroupContainerChildNode *)list->data;
 			gtk_object_set (GTK_OBJECT(child_node->child),
-					"table_selection_model", etgc->table_selection_model,
+					"selection_model", etgc->selection_model,
 					NULL);
 		}
 		break;
 
-	case ARG_TABLE_DRAW_GRID:
-		etgc->draw_grid = GTK_VALUE_BOOL (*arg);
+	case ARG_TABLE_HORIZONTAL_DRAW_GRID:
+		etgc->horizontal_draw_grid = GTK_VALUE_BOOL (*arg);
 		for (list = etgc->children; list; list = g_list_next (list)) {
 			ETableGroupContainerChildNode *child_node = (ETableGroupContainerChildNode *)list->data;
 			gtk_object_set (GTK_OBJECT(child_node->child),
-					"drawgrid", GTK_VALUE_BOOL (*arg),
+					"horizontal_draw_grid", GTK_VALUE_BOOL (*arg),
+					NULL);
+		}
+		break;
+
+	case ARG_TABLE_VERTICAL_DRAW_GRID:
+		etgc->vertical_draw_grid = GTK_VALUE_BOOL (*arg);
+		for (list = etgc->children; list; list = g_list_next (list)) {
+			ETableGroupContainerChildNode *child_node = (ETableGroupContainerChildNode *)list->data;
+			gtk_object_set (GTK_OBJECT(child_node->child),
+					"vertical_draw_grid", GTK_VALUE_BOOL (*arg),
 					NULL);
 		}
 		break;
@@ -726,6 +807,7 @@ etgc_class_init (GtkObjectClass *object_class)
 	etgc_parent_class = gtk_type_class (PARENT_TYPE);
 
 	e_group_class->add = etgc_add;
+	e_group_class->add_array = etgc_add_array;
 	e_group_class->add_all = etgc_add_all;
 	e_group_class->remove = etgc_remove;
 	e_group_class->increment  = etgc_increment;
@@ -736,14 +818,16 @@ etgc_class_init (GtkObjectClass *object_class)
 	e_group_class->get_printable = etgc_get_printable;
 	e_group_class->compute_location = etgc_compute_location;
 
-	gtk_object_add_arg_type ("ETableGroupContainer::drawgrid", GTK_TYPE_BOOL,
-				 GTK_ARG_WRITABLE, ARG_TABLE_DRAW_GRID);
+	gtk_object_add_arg_type ("ETableGroupContainer::horizontal_draw_grid", GTK_TYPE_BOOL,
+				 GTK_ARG_WRITABLE, ARG_TABLE_HORIZONTAL_DRAW_GRID);
+	gtk_object_add_arg_type ("ETableGroupContainer::vertical_draw_grid", GTK_TYPE_BOOL,
+				 GTK_ARG_WRITABLE, ARG_TABLE_VERTICAL_DRAW_GRID);
 	gtk_object_add_arg_type ("ETableGroupContainer::drawfocus", GTK_TYPE_BOOL,
 				 GTK_ARG_WRITABLE, ARG_TABLE_DRAW_FOCUS);
 	gtk_object_add_arg_type ("ETableGroupContainer::cursor_mode", GTK_TYPE_INT,
 				 GTK_ARG_WRITABLE, ARG_CURSOR_MODE);
-	gtk_object_add_arg_type ("ETableGroupContainer::table_selection_model", GTK_TYPE_OBJECT,
-				 GTK_ARG_WRITABLE, ARG_TABLE_SELECTION_MODEL);
+	gtk_object_add_arg_type ("ETableGroupContainer::selection_model", E_SELECTION_MODEL_TYPE,
+				 GTK_ARG_WRITABLE, ARG_SELECTION_MODEL);
 	gtk_object_add_arg_type ("ETableGroupContainer::length_threshold", GTK_TYPE_INT,
 				 GTK_ARG_WRITABLE, ARG_LENGTH_THRESHOLD);
 
@@ -846,11 +930,12 @@ etgc_init (GtkObject *object)
 	
 	e_canvas_item_set_reflow_callback (GNOME_CANVAS_ITEM(object), etgc_reflow);
 
-	container->draw_grid = 1;
+	container->horizontal_draw_grid = 1;
+	container->vertical_draw_grid = 1;
 	container->draw_focus = 1;
-	container->cursor_mode = E_TABLE_CURSOR_SIMPLE;
+	container->cursor_mode = E_CURSOR_SIMPLE;
 	container->length_threshold = -1;
-	container->table_selection_model = NULL;
+	container->selection_model = NULL;
 }
 
 E_MAKE_TYPE (e_table_group_container, "ETableGroupContainer", ETableGroupContainer, etgc_class_init, etgc_init, PARENT_TYPE);
