@@ -4,9 +4,11 @@
 
 #include <libgnomeui/gnome-canvas.h>
 #include <gal/e-table/e-table-group.h>
-#include <gal/e-table/e-table-sorted-variable.h>
+#include <gal/e-table/e-table-subset.h>
 #include <gal/e-table/e-table-item.h>
 
+BEGIN_GNOME_DECLS
+
 #define E_TABLE_GROUP_LEAF_TYPE        (e_table_group_leaf_get_type ())
 #define E_TABLE_GROUP_LEAF(o)          (GTK_CHECK_CAST ((o), E_TABLE_GROUP_LEAF_TYPE, ETableGroupLeaf))
 #define E_TABLE_GROUP_LEAF_CLASS(k)    (GTK_CHECK_CLASS_CAST((k), E_TABLE_GROUP_LEAF_TYPE, ETableGroupLeafClass))
@@ -25,15 +27,17 @@ typedef struct {
 	gdouble width;
 	gdouble minimum_width;
 
-	ETableSubsetVariable *subset;
-
 	int length_threshold;
 
-	guint draw_grid : 1;
-	guint draw_focus : 1;
-	ETableCursorMode cursor_mode;
+	ETableSubset *ets;
+	guint is_grouped : 1;
 
-	ETableSelectionModel *table_selection_model;
+	guint horizontal_draw_grid : 1;
+	guint vertical_draw_grid : 1;
+	guint draw_focus : 1;
+	ECursorMode cursor_mode;
+
+	ESelectionModel *selection_model;
 } ETableGroupLeaf;
 
 typedef struct {
@@ -48,5 +52,7 @@ ETableGroup *e_table_group_leaf_new       (GnomeCanvasGroup *parent,
 GtkType      e_table_group_leaf_get_type  (void);
 
 
+END_GNOME_DECLS
+
 #endif /* _E_TABLE_GROUP_LEAF_H_ */
 
