@@ -10,6 +10,8 @@
 #include <gal/util/e-util.h>
 #include <gal/widgets/e-printable.h>
 
+BEGIN_GNOME_DECLS
+
 #define E_TABLE_GROUP_TYPE        (e_table_group_get_type ())
 #define E_TABLE_GROUP(o)          (GTK_CHECK_CAST ((o), E_TABLE_GROUP_TYPE, ETableGroup))
 #define E_TABLE_GROUP_CLASS(k)    (GTK_CHECK_CLASS_CAST((k), E_TABLE_GROUP_TYPE, ETableGroupClass))
@@ -54,6 +56,7 @@ typedef struct {
 
 	/* Virtual functions. */
 	void        (*add)                   (ETableGroup *etg, gint row);
+	void        (*add_array)             (ETableGroup *etg, const int *array, int count);
 	void        (*add_all)               (ETableGroup *etg);
 	gboolean    (*remove)                (ETableGroup *etg, gint row);
 	gint        (*row_count)             (ETableGroup *etg);
@@ -64,12 +67,16 @@ typedef struct {
 	gint        (*get_focus_column)      (ETableGroup *etg);
 	EPrintable *(*get_printable)         (ETableGroup *etg);
 	void        (*compute_location)      (ETableGroup *etg, int *x, int *y, int *row, int *col);
+	void        (*get_position)          (ETableGroup *etg, int *x, int *y, int *row, int *col);
 
 } ETableGroupClass;
 
 /* Virtual functions */
 void          e_table_group_add               (ETableGroup       *etg,
 					       gint               row);
+void          e_table_group_add_array         (ETableGroup       *etg,
+					       const int         *array,
+					       int                count);
 void          e_table_group_add_all           (ETableGroup       *etg);
 gboolean      e_table_group_remove            (ETableGroup       *etg,
 					       gint               row);
@@ -92,7 +99,11 @@ void          e_table_group_compute_location  (ETableGroup       *etg,
 					       int               *y,
 					       int               *row,
 					       int               *col);
-
+void          e_table_group_get_position      (ETableGroup       *etg,
+					       int               *x,
+					       int               *y,
+					       int               *row,
+					       int               *col);
 ETableGroup  *e_table_group_new               (GnomeCanvasGroup  *parent,
 					       ETableHeader      *full_header,
 					       ETableHeader      *header,
@@ -133,4 +144,6 @@ void          e_table_group_apply_to_leafs    (ETableGroup       *etg,
 					       ETableGroupLeafFn  fn,
 					       void              *closure);
 
+END_GNOME_DECLS
+
 #endif /* _E_TABLE_GROUP_H_ */
