@@ -819,6 +819,9 @@ ethi_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int width
 		if (x2 < x)
 			continue;
 
+		if (x2 <= x1)
+			continue;
+
 		e_table_header_draw_button (drawable, ecol,
 					    GTK_WIDGET (canvas)->style, ethi->font,
 					    GTK_WIDGET_STATE (canvas),
@@ -886,7 +889,7 @@ set_cursor (ETableHeaderItem *ethi, int pos)
 {
 	int col;
 	GtkWidget *canvas = GTK_WIDGET (GNOME_CANVAS_ITEM (ethi)->canvas);
-	gboolean resizeable = FALSE;
+	gboolean resizable = FALSE;
 		
 	/* We might be invoked before we are realized */
 	if (!canvas->window)
@@ -897,7 +900,7 @@ set_cursor (ETableHeaderItem *ethi, int pos)
 		ETableCol *ecol = e_table_header_get_column (ethi->eth, col);
 
 		/* Last column is not resizable */
-		if (ecol->resizeable && col != last_col) {
+		if (ecol->resizable && col != last_col) {
 			int c = col + 1;
 
 			/* Column is not resizable if all columns after it
@@ -906,15 +909,15 @@ set_cursor (ETableHeaderItem *ethi, int pos)
 				ETableCol *ecol2;
 
 				ecol2 = e_table_header_get_column (ethi->eth, c);
-				if (ecol2->resizeable) {
-					resizeable = TRUE;
+				if (ecol2->resizable) {
+					resizable = TRUE;
 					break;
 				}
 			}
 		}
 	}
 	
-	if (resizeable)
+	if (resizable)
 		e_cursor_set (canvas->window, E_CURSOR_SIZE_X);
 	else
 		gdk_window_set_cursor (canvas->window, NULL);
@@ -1221,14 +1224,10 @@ config_destroyed (GtkObject *object, ETableHeaderItem *ethi)
 }
 
 static void
-apply_changes (GnomePropertyBox *pbox, gint page_num, ETableHeaderItem *ethi)
+apply_changes (ETableConfig *config, ETableHeaderItem *ethi)
 {
-	ETableConfig *config = E_TABLE_CONFIG (ethi->config);
 	char *state = e_table_state_save_to_string (config->state);
 
-	if (page_num != -1)
-		return;
-
 	e_table_set_state (ethi->table, state);
 	g_free (state);
 }
@@ -1251,9 +1250,7 @@ ethi_popup_customize_view(GtkWidget *widget, EthiHeaderInfo *info)
 			GTK_OBJECT (ethi->config), "destroy",
 			GTK_SIGNAL_FUNC (config_destroyed), ethi);
 		gtk_signal_connect (
-			GTK_OBJECT (
-				E_TABLE_CONFIG (ethi->config)->dialog_toplevel),
-			"apply",
+			GTK_OBJECT (ethi->config), "changed",
 			GTK_SIGNAL_FUNC (apply_changes), ethi);
 	}
 }
@@ -1432,7 +1429,7 @@ ethi_event (GnomeCanvasItem *item, GdkEvent *e)
 				 */
 			ecol = e_table_header_get_column (ethi->eth, col);
 			
-			if (!ecol->resizeable)
+			if (!ecol->resizable)
 				break;
 			ethi->resize_col = col;
 			ethi->resize_start_pos = start - ecol->width;
@@ -1542,7 +1539,7 @@ ethi_class_init (GtkObjectClass *object_class)
 				object_class->type,
 				GTK_SIGNAL_OFFSET (ETableHeaderItemClass, button_pressed),
 				gtk_marshal_NONE__POINTER,
-				GTK_TYPE_NONE, 1, GTK_TYPE_POINTER);
+				GTK_TYPE_NONE, 1, GTK_TYPE_GDK_EVENT);
 		
 	gtk_object_class_add_signals (object_class, ethi_signals, LAST_SIGNAL);
 }
