@@ -149,7 +149,8 @@ eth_destroy (GtkObject *object)
 	for (i = cols - 1; i >= 0; i--){
 		eth_do_remove (eth, i, TRUE);
 	}
-	
+	g_free (eth->columns);
+
 	if (e_table_header_parent_class->destroy)
 		e_table_header_parent_class->destroy (object);
 }
@@ -619,7 +620,7 @@ eth_set_size (ETableHeader *eth, int idx, int size)
 	g_return_if_fail (idx < eth->col_count);
 	
 	/* If this column is not resizable, don't do anything. */
-	if (!eth->columns[idx]->resizeable)
+	if (!eth->columns[idx]->resizable)
 		return;
 
 	expansion = 0;
@@ -642,7 +643,7 @@ eth_set_size (ETableHeader *eth, int idx, int size)
 	 */
 	for (; i < eth->col_count; i++) {
 		min_width += eth->columns[i]->min_width + eth->width_extras;
-		if (eth->columns[i]->resizeable) {
+		if (eth->columns[i]->resizable) {
 			expansion += eth->columns[i]->expansion;
 			expandable_count ++;
 		}
@@ -703,7 +704,7 @@ eth_set_size (ETableHeader *eth, int idx, int size)
 	 */
 	if (old_expansion == 0) {
 		for (i = idx + 1; i < eth->col_count; i++) {
-			if (eth->columns[idx]->resizeable) {
+			if (eth->columns[idx]->resizable) {
 				/* expandable_count != 0 by (1) */
 				eth->columns[i]->expansion = expansion / expandable_count;
 			}
@@ -714,7 +715,7 @@ eth_set_size (ETableHeader *eth, int idx, int size)
 	/* Remove from total_extra the amount used for this column. */
 	total_extra -= size - (eth->columns[idx]->min_width + eth->width_extras);
 	for (i = idx + 1; i < eth->col_count; i++) {
-		if (eth->columns[idx]->resizeable) {
+		if (eth->columns[idx]->resizable) {
 			/* old_expansion != 0 by (2) */
 			eth->columns[i]->expansion *= expansion / old_expansion;
 		}
@@ -769,9 +770,9 @@ eth_calc_widths (ETableHeader *eth)
 	expansion = 0;
 	for (i = 0; i < eth->col_count; i++) {
 		extra -= eth->columns[i]->min_width + eth->width_extras;
-		if (eth->columns[i]->resizeable && eth->columns[i]->expansion > 0)
+		if (eth->columns[i]->resizable && eth->columns[i]->expansion > 0)
 			last_resizable = i;
-		expansion += eth->columns[i]->resizeable ? eth->columns[i]->expansion : 0;
+		expansion += eth->columns[i]->resizable ? eth->columns[i]->expansion : 0;
 		eth->columns[i]->width = eth->columns[i]->min_width + eth->width_extras;
 	}
 	if (eth->sort_info)
@@ -779,7 +780,7 @@ eth_calc_widths (ETableHeader *eth)
 	if (expansion == 0 || extra <= 0)
 		return;
 	for (i = 0; i < last_resizable; i++) {
-		next_position += extra * (eth->columns[i]->resizeable ? eth->columns[i]->expansion : 0)/expansion;
+		next_position += extra * (eth->columns[i]->resizable ? eth->columns[i]->expansion : 0)/expansion;
 		eth->columns[i]->width += next_position - last_position;
 		last_position = next_position;
 	}
@@ -789,6 +790,26 @@ eth_calc_widths (ETableHeader *eth)
 	gtk_signal_emit (GTK_OBJECT (eth), eth_signals [DIMENSION_CHANGE]);
 }
 
+void
+e_table_header_update_horizontal (ETableHeader *eth)
+{
+	int i;
+	int cols;
+
+	cols = eth->col_count;
+
+	for (i = 0; i < cols; i++) {
+		int width;
+		
+		gtk_signal_emit_by_name (GTK_OBJECT (eth),
+					 "request_width",
+					 i, &width);
+		eth->columns[i]->min_width = width + 10;
+		eth->columns[i]->expansion = 1;
+	}
+	enqueue(eth, -1, eth->nominal_width);
+}
+
 GtkType
 e_table_header_get_type (void)
 {
@@ -811,4 +832,3 @@ e_table_header_get_type (void)
 
 	return type;
 }
-
