@@ -5,11 +5,13 @@
 #include <libgnomeui/gnome-canvas.h>
 #include <gal/e-table/e-table-model.h>
 #include <gal/e-table/e-table-header.h>
-#include <gal/e-table/e-table-selection-model.h>
 #include <gal/e-table/e-table-defines.h>
 #include <gal/e-table/e-table-tooltip.h>
+#include <gal/widgets/e-selection-model.h>
 #include <gal/widgets/e-printable.h>
 
+BEGIN_GNOME_DECLS
+
 #define E_TABLE_ITEM_TYPE        (e_table_item_get_type ())
 #define E_TABLE_ITEM(o)          (GTK_CHECK_CAST ((o), E_TABLE_ITEM_TYPE, ETableItem))
 #define E_TABLE_ITEM_CLASS(k)    (GTK_CHECK_CLASS_CAST((k), E_TABLE_ITEM_TYPE, ETableItemClass))
@@ -22,7 +24,7 @@ typedef struct {
 	ETableHeader    *header;
 
 	ETableModel     *source_model;
-	ETableSelectionModel *selection;
+	ESelectionModel *selection;
 
 	int              x1, y1;
 	int              minimum_width, width, height;
@@ -39,8 +41,8 @@ typedef struct {
 	int              table_model_change_id;
 	int              table_model_row_change_id;
 	int              table_model_cell_change_id;
-	int              table_model_row_inserted_id;
-	int              table_model_row_deleted_id;
+	int              table_model_rows_inserted_id;
+	int              table_model_rows_deleted_id;
 
 	int              selection_change_id;
 	int              cursor_change_id;
@@ -51,7 +53,8 @@ typedef struct {
 	GdkGC           *focus_gc;
 	GdkBitmap       *stipple;
 
-	guint 		 draw_grid:1;
+	guint 		 horizontal_draw_grid:1;
+	guint 		 vertical_draw_grid:1;
 	guint 		 draw_focus:1;
 	guint 		 renderers_can_change_size:1;
 	guint 		 cell_views_realized:1;
@@ -81,7 +84,7 @@ typedef struct {
 	int              length_threshold;
 	
 	gint             row_guess;
-	ETableCursorMode cursor_mode;
+	ECursorMode      cursor_mode;
 
 	/*
 	 * During editing
@@ -103,7 +106,7 @@ typedef struct {
 
 	void        (*cursor_change) (ETableItem *eti, int row);
 	void        (*cursor_activated) (ETableItem *eti, int row);
-	void        (*double_click)  (ETableItem *eti, int row);
+	void        (*double_click)  (ETableItem *eti, int row, int col, GdkEvent *event);
 	gint        (*right_click)   (ETableItem *eti, int row, int col, GdkEvent *event);
 	gint        (*click)   (ETableItem *eti, int row, int col, GdkEvent *event);
 	gint        (*key_press)     (ETableItem *eti, int row, int col, GdkEvent *event);
@@ -138,5 +141,10 @@ void        e_table_item_compute_location    (ETableItem        *eti,
 					      int               *row,
 					      int               *col);
 
+int	    e_table_item_row_diff	     (ETableItem	*eti,
+					      int		 start_row,
+					      int		 end_row);
+
+END_GNOME_DECLS
 
 #endif /* _E_TABLE_ITEM_H_ */
