@@ -9,13 +9,15 @@
  * (C) 2000, 2001 Ximian, Inc.
  */
 #include <config.h>
+
+#include "e-table-specification.h"
+
 #include <stdlib.h>
 #include <gtk/gtksignal.h>
 #include <gnome-xml/parser.h>
 #include <gnome-xml/xmlmemory.h>
 #include "gal/util/e-util.h"
 #include "gal/util/e-xml-utils.h"
-#include "e-table-specification.h"
 
 #define PARENT_TYPE (gtk_object_get_type ())
 
@@ -61,9 +63,12 @@ etsp_init (ETableSpecification *etsp)
 
 	etsp->no_headers            = FALSE;
 	etsp->click_to_add          = FALSE;
-	etsp->draw_grid             = FALSE;
+	etsp->horizontal_draw_grid  = FALSE;
+	etsp->vertical_draw_grid    = FALSE;
 	etsp->draw_focus            = TRUE;
-	etsp->cursor_mode           = E_TABLE_CURSOR_SIMPLE;
+	etsp->horizontal_scrolling  = FALSE;
+
+	etsp->cursor_mode           = E_CURSOR_SIMPLE;
 	etsp->selection_mode        = GTK_SELECTION_MULTIPLE;
 
 	etsp->click_to_add_message  = NULL;
@@ -158,26 +163,33 @@ e_table_specification_load_from_node (ETableSpecification *specification,
 
 	specification->no_headers = e_xml_get_bool_prop_by_name (node, "no-headers");
 	specification->click_to_add = e_xml_get_bool_prop_by_name (node, "click-to-add");
-	specification->draw_grid = e_xml_get_bool_prop_by_name (node, "draw-grid");
+	specification->horizontal_draw_grid = e_xml_get_bool_prop_by_name (node, "horizontal-draw-grid");
+	specification->vertical_draw_grid = e_xml_get_bool_prop_by_name (node, "vertical-draw-grid");
+	if (e_xml_get_bool_prop_by_name_with_default(node, "draw-grid", TRUE) ==
+	    e_xml_get_bool_prop_by_name_with_default(node, "draw-grid", FALSE)) {
+		specification->horizontal_draw_grid =
+			specification->vertical_draw_grid = e_xml_get_bool_prop_by_name (node, "draw-grid");
+	}
 	specification->draw_focus = e_xml_get_bool_prop_by_name_with_default (node, "draw-focus", TRUE);
+	specification->horizontal_scrolling = e_xml_get_bool_prop_by_name_with_default (node, "horizontal-scrolling", FALSE);
 
 	specification->selection_mode = GTK_SELECTION_MULTIPLE;
 	temp = e_xml_get_string_prop_by_name (node, "selection-mode");
-	if (temp && !strcasecmp (temp, "single")) {
+	if (temp && !g_strcasecmp (temp, "single")) {
 		specification->selection_mode = GTK_SELECTION_SINGLE;
-	} else if (temp && !strcasecmp (temp, "browse")) {
+	} else if (temp && !g_strcasecmp (temp, "browse")) {
 		specification->selection_mode = GTK_SELECTION_BROWSE;
-	} else if (temp && !strcasecmp (temp, "extended")) {
+	} else if (temp && !g_strcasecmp (temp, "extended")) {
 		specification->selection_mode = GTK_SELECTION_EXTENDED;
 	}
 	g_free (temp);
 
-	specification->cursor_mode = E_TABLE_CURSOR_SIMPLE;
+	specification->cursor_mode = E_CURSOR_SIMPLE;
 	temp = e_xml_get_string_prop_by_name (node, "cursor-mode");
-	if (temp && !strcasecmp (temp, "line")) {
-		specification->cursor_mode = E_TABLE_CURSOR_LINE;
-	} else 	if (temp && !strcasecmp (temp, "spreadsheet")) {
-		specification->cursor_mode = E_TABLE_CURSOR_SPREADSHEET;
+	if (temp && !g_strcasecmp (temp, "line")) {
+		specification->cursor_mode = E_CURSOR_LINE;
+	} else 	if (temp && !g_strcasecmp (temp, "spreadsheet")) {
+		specification->cursor_mode = E_CURSOR_SPREADSHEET;
 	}
 	g_free (temp);
 	g_free (specification->click_to_add_message);
@@ -296,8 +308,10 @@ e_table_specification_save_to_node (ETableSpecification *specification,
 	node = xmlNewNode (NULL, "ETableSpecification");
 	e_xml_set_bool_prop_by_name (node, "no-headers", specification->no_headers);
 	e_xml_set_bool_prop_by_name (node, "click-to-add", specification->click_to_add);
-	e_xml_set_bool_prop_by_name (node, "draw-grid", specification->draw_grid);
+	e_xml_set_bool_prop_by_name (node, "horizontal-draw-grid", specification->horizontal_draw_grid);
+	e_xml_set_bool_prop_by_name (node, "vertical-draw-grid", specification->vertical_draw_grid);
 	e_xml_set_bool_prop_by_name (node, "draw-focus", specification->draw_focus);
+	e_xml_set_bool_prop_by_name (node, "horizontal-scrolling", specification->horizontal_scrolling);
 
 	switch (specification->selection_mode){
 	case GTK_SELECTION_SINGLE:
@@ -311,7 +325,7 @@ e_table_specification_save_to_node (ETableSpecification *specification,
 		s = "extended";
 	}
 	xmlSetProp (node, "selection-mode", s);
-	if (specification->cursor_mode == E_TABLE_CURSOR_LINE)
+	if (specification->cursor_mode == E_CURSOR_LINE)
 		s = "line";
 	else
 		s = "cell";
