@@ -4,10 +4,15 @@
 
 #include <gtk/gtkobject.h>
 #include <gnome-xml/tree.h>
+#include <gal/widgets/e-selection-model.h>
 #include <gal/e-table/e-table-state.h>
 #include <gal/e-table/e-table-column-specification.h>
 #include <gal/e-table/e-table-defines.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
+
 #define E_TABLE_SPECIFICATION_TYPE        (e_table_specification_get_type ())
 #define E_TABLE_SPECIFICATION(o)          (GTK_CHECK_CAST ((o), E_TABLE_SPECIFICATION_TYPE, ETableSpecification))
 #define E_TABLE_SPECIFICATION_CLASS(k)    (GTK_CHECK_CLASS_CAST((k), E_TABLE_SPECIFICATION_TYPE, ETableSpecificationClass))
@@ -22,10 +27,12 @@ typedef struct {
 
 	guint no_headers : 1;
 	guint click_to_add : 1;
-	guint draw_grid : 1;
+	guint horizontal_draw_grid : 1;
+	guint vertical_draw_grid : 1;
 	guint draw_focus : 1;
+	guint horizontal_scrolling : 1;
 	GtkSelectionMode selection_mode;
-	ETableCursorMode cursor_mode;
+	ECursorMode cursor_mode;
 
 	char *click_to_add_message;
 } ETableSpecification;
@@ -51,4 +58,8 @@ xmlNode             *e_table_specification_save_to_node      (ETableSpecificatio
 							      xmlDoc              *doc);
 ETableSpecification *e_table_specification_duplicate         (ETableSpecification *spec);
 
+#ifdef __cplusplus
+}
+#endif /* __cplusplus */
+
 #endif /* _E_TABLE_SPECIFICATION_H_ */
