@@ -11,12 +11,14 @@
 #include <config.h>
 #include <stdlib.h>
 #include <gtk/gtksignal.h>
+#include <gtk/gtkobject.h>
 #include <gnome-xml/parser.h>
 #include <gnome-xml/xmlmemory.h>
 #include "gal/util/e-util.h"
 #include "gal/util/e-xml-utils.h"
 #include "e-table-state.h"
 
+
 #define PARENT_TYPE (gtk_object_get_type())
 
 #define STATE_VERSION 0.1
@@ -28,9 +30,17 @@ etst_destroy (GtkObject *object)
 {
 	ETableState *etst = E_TABLE_STATE (object);
 
-	if (etst->columns)
+	gtk_object_unref (GTK_OBJECT (etst->sort_info));
+	if (etst->columns) {
 		g_free (etst->columns);
+		etst->columns = NULL;
+	}
 
+	if (etst->expansions) {
+		g_free (etst->expansions);
+		etst->expansions = NULL;
+	}
+	
 	GTK_OBJECT_CLASS (etst_parent_class)->destroy (object);
 }
 
@@ -45,6 +55,8 @@ etst_class_init (GtkObjectClass *klass)
 static void
 etst_init (ETableState *state)
 {
+	GTK_OBJECT_UNSET_FLAGS (GTK_OBJECT (state), GTK_FLOATING);
+
 	state->columns = NULL;
 	state->expansions = NULL;
 	state->sort_info = e_table_sort_info_new();
@@ -229,4 +241,3 @@ e_table_state_duplicate (ETableState *state)
 
 	return new_state;
 }
-
