@@ -13,13 +13,16 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdio.h>
-#include "gal/util/e-i18n.h"
-#include <libgnomeui/gnome-canvas.h>
+#include <gdk/gdkkeysyms.h>
 #include <gtk/gtksignal.h>
+#include <libgnomeui/gnome-canvas.h>
+#include <libgnomeui/gnome-canvas-rect-ellipse.h>
 
+#include "gal/util/e-i18n.h"
 #include "gal/util/e-util.h"
 #include "gal/widgets/e-canvas.h"
 #include "gal/widgets/e-canvas-vbox.h"
+#include "gal/widgets/e-unicode.h"
 #include "e-table.h"
 #include "e-table-header-item.h"
 #include "e-table-header-utils.h"
@@ -32,6 +35,8 @@
 #include "e-table-state.h"
 #include "e-table-column-specification.h"
 
+#include "e-table-utils.h"
+
 #define COLUMN_HEADER_HEIGHT 16
 
 #define PARENT_TYPE gtk_table_get_type ()
@@ -118,23 +123,42 @@ static gint e_table_drag_source_event_cb (GtkWidget      *widget,
 
 static gint et_focus (GtkContainer *container, GtkDirectionType direction);
 
+static void
+et_disconnect_model (ETable *et)
+{
+	if (et->model == NULL)
+		return;
+
+	if (et->table_model_change_id != 0)
+		gtk_signal_disconnect (GTK_OBJECT (et->model),
+				       et->table_model_change_id);
+	if (et->table_row_change_id != 0)
+		gtk_signal_disconnect (GTK_OBJECT (et->model),
+				       et->table_row_change_id);
+	if (et->table_cell_change_id != 0)
+		gtk_signal_disconnect (GTK_OBJECT (et->model),
+				       et->table_cell_change_id);
+	if (et->table_rows_inserted_id != 0)
+		gtk_signal_disconnect (GTK_OBJECT (et->model),
+				       et->table_rows_inserted_id);
+	if (et->table_rows_deleted_id != 0)
+		gtk_signal_disconnect (GTK_OBJECT (et->model),
+				       et->table_rows_deleted_id);
+
+	et->table_model_change_id = 0;
+	et->table_row_change_id = 0;
+	et->table_cell_change_id = 0;
+	et->table_rows_inserted_id = 0;
+	et->table_rows_deleted_id = 0;
+}
 
 static void
 et_destroy (GtkObject *object)
 {
 	ETable *et = E_TABLE (object);
 
+	et_disconnect_model (et);
 
-	gtk_signal_disconnect (GTK_OBJECT (et->model),
-			       et->table_model_change_id);
-	gtk_signal_disconnect (GTK_OBJECT (et->model),
-			       et->table_row_change_id);
-	gtk_signal_disconnect (GTK_OBJECT (et->model),
-			       et->table_cell_change_id);
-	gtk_signal_disconnect (GTK_OBJECT (et->model),
-			       et->table_row_inserted_id);
-	gtk_signal_disconnect (GTK_OBJECT (et->model),
-			       et->table_row_deleted_id);
 	if (et->group_info_change_id)
 		gtk_signal_disconnect (GTK_OBJECT (et->sort_info),
 				       et->group_info_change_id);
@@ -181,14 +205,17 @@ e_table_init (GtkObject *object)
 	e_table->group_info_change_id = 0;
 	e_table->reflow_idle_id = 0;
 
-	e_table->draw_grid = 1;
+	e_table->horizontal_draw_grid = 1;
+	e_table->vertical_draw_grid = 1;
 	e_table->draw_focus = 1;
-	e_table->cursor_mode = E_TABLE_CURSOR_SIMPLE;
+	e_table->cursor_mode = E_CURSOR_SIMPLE;
 	e_table->length_threshold = 200;
 
 	e_table->need_rebuild = 0;
 	e_table->rebuild_idle_id = 0;
 
+	e_table->horizontal_scrolling = FALSE;
+
 	e_table->click_to_add_message = NULL;
 
 	e_table->drag_get_data_row = -1;
@@ -232,13 +259,32 @@ et_focus (GtkContainer *container, GtkDirectionType direction)
 	return gtk_container_focus (GTK_CONTAINER (e_table->table_canvas), direction);
 }
 
+static void
+set_header_canvas_width (ETable *e_table)
+{
+	double oldwidth, oldheight, width;
+
+	if (!(e_table->header_item && e_table->header_canvas && e_table->table_canvas))
+		return;
+
+	gnome_canvas_get_scroll_region (GNOME_CANVAS (e_table->table_canvas),
+					NULL, NULL, &width, NULL);
+	gnome_canvas_get_scroll_region (GNOME_CANVAS (e_table->header_canvas),
+					NULL, NULL, &oldwidth, &oldheight);
+
+	if (oldwidth != width ||
+	    oldheight != E_TABLE_HEADER_ITEM (e_table->header_item)->height - 1)
+		gnome_canvas_set_scroll_region (
+						GNOME_CANVAS (e_table->header_canvas),
+						0, 0, width, /*  COLUMN_HEADER_HEIGHT - 1 */
+						E_TABLE_HEADER_ITEM (e_table->header_item)->height - 1);
+
+}
+
 static void
 header_canvas_size_allocate (GtkWidget *widget, GtkAllocation *alloc, ETable *e_table)
 {
-	gnome_canvas_set_scroll_region (
-		GNOME_CANVAS (e_table->header_canvas),
-		0, 0, alloc->width - 1, /*  COLUMN_HEADER_HEIGHT - 1 */
-		E_TABLE_HEADER_ITEM (e_table->header_item)->height - 1);
+	set_header_canvas_width (e_table);
 
 	/* When the header item is created ->height == 0,
 	   as the font is only created when everything is realized.
@@ -253,9 +299,12 @@ header_canvas_size_allocate (GtkWidget *widget, GtkAllocation *alloc, ETable *e_
 static void
 sort_info_changed (ETableSortInfo *info, ETable *et)
 {
-	et->need_rebuild = TRUE;
-	if (!et->rebuild_idle_id)
-		et->rebuild_idle_id = g_idle_add_full (20, changed_idle, et, NULL);
+	gboolean will_be_grouped = e_table_sort_info_grouping_get_count(info) > 0;
+	if (et->is_grouped || will_be_grouped) {
+		et->need_rebuild = TRUE;
+		if (!et->rebuild_idle_id)
+			et->rebuild_idle_id = g_idle_add_full (20, changed_idle, et, NULL);
+	}
 }
 
 static void
@@ -294,6 +343,7 @@ table_canvas_reflow_idle (ETable *e_table)
 {
 	gdouble height, width;
 	gdouble item_height;
+	gdouble oldheight, oldwidth;
 	GtkAllocation *alloc = &(GTK_WIDGET (e_table->table_canvas)->allocation);
 
 	gtk_object_get (GTK_OBJECT (e_table->canvas_vbox),
@@ -304,11 +354,17 @@ table_canvas_reflow_idle (ETable *e_table)
 	height = MAX ((int)height, alloc->height);
 	width = MAX((int)width, alloc->width);
 	/* I have no idea why this needs to be -1, but it works. */
-	gnome_canvas_set_scroll_region (
-		GNOME_CANVAS (e_table->table_canvas),
-		0, 0, width - 1, height - 1);
+	gnome_canvas_get_scroll_region (GNOME_CANVAS (e_table->table_canvas),
+					NULL, NULL, &oldwidth, &oldheight);
+
+	if (oldwidth != width - 1 ||
+	    oldheight != height - 1) {
+		gnome_canvas_set_scroll_region (GNOME_CANVAS (e_table->table_canvas),
+						0, 0, width - 1, height - 1);
+		set_header_canvas_width (e_table);
+	}
 	gtk_object_set (GTK_OBJECT (e_table->white_item),
-			"y1", item_height + 1,
+			"y1", item_height,
 			"x2", width,
 			"y2", height,
 			NULL);
@@ -358,7 +414,7 @@ static void
 click_to_add_cursor_change (ETableClickToAdd *etcta, int row, int col, ETable *et)
 {
 	if (et->cursor_loc == E_TABLE_CURSOR_LOC_TABLE) {
-		e_table_selection_model_clear(et->selection);
+		e_selection_model_clear(E_SELECTION_MODEL (et->selection));
 	}
 	et->cursor_loc = E_TABLE_CURSOR_LOC_ETCTA;
 }
@@ -464,55 +520,6 @@ group_key_press (ETableGroup *etg, int row, int col, GdkEvent *event, ETable *et
 	return return_val;
 }
 
-static gboolean
-changed_idle (gpointer data)
-{
-	ETable *et = E_TABLE (data);
-
-	if (et->need_rebuild) {
-		gtk_object_destroy (GTK_OBJECT (et->group));
-		et->group = e_table_group_new (GNOME_CANVAS_GROUP (et->canvas_vbox),
-					       et->full_header,
-					       et->header,
-					       et->model,
-					       et->sort_info,
-					       0);
-		e_canvas_vbox_add_item(E_CANVAS_VBOX(et->canvas_vbox), GNOME_CANVAS_ITEM(et->group));
-		gnome_canvas_item_set(GNOME_CANVAS_ITEM(et->group),
-				      "drawgrid", et->draw_grid,
-				      "drawfocus", et->draw_focus,
-				      "cursor_mode", et->cursor_mode,
-				      "length_threshold", et->length_threshold,
-				      "table_selection_model", et->selection,
-				      NULL);
-		gtk_signal_connect (GTK_OBJECT (et->group), "cursor_change",
-				    GTK_SIGNAL_FUNC (group_cursor_change), et);
-		gtk_signal_connect (GTK_OBJECT (et->group), "cursor_activated",
-				    GTK_SIGNAL_FUNC (group_cursor_activated), et);
-		gtk_signal_connect (GTK_OBJECT (et->group), "double_click",
-				    GTK_SIGNAL_FUNC (group_double_click), et);
-		gtk_signal_connect (GTK_OBJECT (et->group), "right_click",
-				    GTK_SIGNAL_FUNC (group_right_click), et);
-		gtk_signal_connect (GTK_OBJECT (et->group), "click",
-				    GTK_SIGNAL_FUNC (group_click), et);
-		gtk_signal_connect (GTK_OBJECT (et->group), "key_press",
-				    GTK_SIGNAL_FUNC (group_key_press), et);
-		e_table_fill_table (et, et->model);
-
-		gtk_object_set (GTK_OBJECT (et->canvas_vbox),
-				"width", (double) GTK_WIDGET (et->table_canvas)->allocation.width,
-				NULL);
-
-		if (GTK_WIDGET_REALIZED(et->table_canvas))
-			table_canvas_size_allocate (GTK_WIDGET(et->table_canvas), &GTK_WIDGET(et->table_canvas)->allocation, et);
-	}
-
-	et->need_rebuild = 0;
-	et->rebuild_idle_id = 0;
-
-	return FALSE;
-}
-
 static void
 et_table_model_changed (ETableModel *model, ETable *et)
 {
@@ -527,6 +534,8 @@ et_table_row_changed (ETableModel *table_model, int row, ETable *et)
 	if (!et->need_rebuild) {
 		if (e_table_group_remove (et->group, row))
 			e_table_group_add (et->group, row);
+		if (et->horizontal_scrolling)
+			e_table_header_update_horizontal(et->header);
 	}
 }
 
@@ -537,28 +546,123 @@ et_table_cell_changed (ETableModel *table_model, int view_col, int row, ETable *
 }
 
 static void
-et_table_row_inserted (ETableModel *table_model, int row, ETable *et)
+et_table_rows_inserted (ETableModel *table_model, int row, int count, ETable *et)
 {
 	/* This number has already been decremented. */
 	int row_count = e_table_model_row_count(table_model);
 	if (!et->need_rebuild) {
-		if (row != row_count - 1)
-			e_table_group_increment(et->group, row, 1);
-		e_table_group_add (et->group, row);
+		int i;
+		if (row != row_count - count)
+			e_table_group_increment(et->group, row, count);
+		for (i = 0; i < count; i++)
+			e_table_group_add (et->group, row);
+		if (et->horizontal_scrolling)
+			e_table_header_update_horizontal(et->header);
 	}
 }
 
 static void
-et_table_row_deleted (ETableModel *table_model, int row, ETable *et)
+et_table_rows_deleted (ETableModel *table_model, int row, int count, ETable *et)
 {
 	int row_count = e_table_model_row_count(table_model);
 	if (!et->need_rebuild) {
-		e_table_group_remove (et->group, row);
+		int i;
+		for (i = 0; i < count; i++)
+			e_table_group_remove (et->group, row);
 		if (row != row_count)
-			e_table_group_decrement(et->group, row, 1);
+			e_table_group_decrement(et->group, row, count);
+		if (et->horizontal_scrolling)
+			e_table_header_update_horizontal(et->header);
 	}
 }
 
+static void
+et_build_groups (ETable *et)
+{
+	gboolean was_grouped = et->is_grouped;
+
+	et->is_grouped = e_table_sort_info_grouping_get_count(et->sort_info) > 0;
+
+	et->group = e_table_group_new (GNOME_CANVAS_GROUP (et->canvas_vbox),
+				       et->full_header,
+				       et->header,
+				       et->model,
+				       et->sort_info,
+				       0);
+	e_canvas_vbox_add_item(E_CANVAS_VBOX(et->canvas_vbox), GNOME_CANVAS_ITEM(et->group));
+	gnome_canvas_item_set(GNOME_CANVAS_ITEM(et->group),
+			      "horizontal_draw_grid", et->horizontal_draw_grid,
+			      "vertical_draw_grid", et->vertical_draw_grid,
+			      "drawfocus", et->draw_focus,
+			      "cursor_mode", et->cursor_mode,
+			      "length_threshold", et->length_threshold,
+			      "selection_model", et->selection,
+			      NULL);
+
+	gtk_signal_connect (GTK_OBJECT (et->group), "cursor_change",
+			    GTK_SIGNAL_FUNC (group_cursor_change), et);
+	gtk_signal_connect (GTK_OBJECT (et->group), "cursor_activated",
+			    GTK_SIGNAL_FUNC (group_cursor_activated), et);
+	gtk_signal_connect (GTK_OBJECT (et->group), "double_click",
+			    GTK_SIGNAL_FUNC (group_double_click), et);
+	gtk_signal_connect (GTK_OBJECT (et->group), "right_click",
+			    GTK_SIGNAL_FUNC (group_right_click), et);
+	gtk_signal_connect (GTK_OBJECT (et->group), "click",
+			    GTK_SIGNAL_FUNC (group_click), et);
+	gtk_signal_connect (GTK_OBJECT (et->group), "key_press",
+			    GTK_SIGNAL_FUNC (group_key_press), et);
+
+
+	if (!(et->is_grouped) && was_grouped)
+		et_disconnect_model (et);
+
+	if (et->is_grouped && (!was_grouped)) {
+		et->table_model_change_id = gtk_signal_connect (GTK_OBJECT (et->model), "model_changed",
+								GTK_SIGNAL_FUNC (et_table_model_changed), et);
+
+		et->table_row_change_id = gtk_signal_connect (GTK_OBJECT (et->model), "model_row_changed",
+							      GTK_SIGNAL_FUNC (et_table_row_changed), et);
+
+		et->table_cell_change_id = gtk_signal_connect (GTK_OBJECT (et->model), "model_cell_changed",
+							       GTK_SIGNAL_FUNC (et_table_cell_changed), et);
+
+		et->table_rows_inserted_id = gtk_signal_connect (GTK_OBJECT (et->model), "model_rows_inserted",
+								GTK_SIGNAL_FUNC (et_table_rows_inserted), et);
+
+		et->table_rows_deleted_id = gtk_signal_connect (GTK_OBJECT (et->model), "model_rows_deleted",
+							       GTK_SIGNAL_FUNC (et_table_rows_deleted), et);
+
+	}
+
+	if (et->is_grouped)
+		e_table_fill_table (et, et->model);
+}
+
+static gboolean
+changed_idle (gpointer data)
+{
+	ETable *et = E_TABLE (data);
+
+	if (et->need_rebuild) {
+		gtk_object_destroy (GTK_OBJECT (et->group));
+		et_build_groups(et);
+		gtk_object_set (GTK_OBJECT (et->canvas_vbox),
+				"width", (double) GTK_WIDGET (et->table_canvas)->allocation.width,
+				NULL);
+
+		if (GTK_WIDGET_REALIZED(et->table_canvas))
+			table_canvas_size_allocate (GTK_WIDGET(et->table_canvas), &GTK_WIDGET(et->table_canvas)->allocation, et);
+	}
+
+	et->need_rebuild = 0;
+	et->rebuild_idle_id = 0;
+
+	if (et->horizontal_scrolling)
+		e_table_header_update_horizontal(et->header);
+
+	return FALSE;
+}
+
 static void
 et_canvas_realize (GtkWidget *canvas, ETable *e_table)
 {
@@ -665,54 +769,7 @@ e_table_setup_table (ETable *e_table, ETableHeader *full_header, ETableHeader *h
 			GTK_SIGNAL_FUNC(click_to_add_cursor_change), e_table);
 	}
 
-	e_table->group = e_table_group_new (
-		GNOME_CANVAS_GROUP (e_table->canvas_vbox),
-		full_header, header,
-		model, e_table->sort_info, 0);
-	e_canvas_vbox_add_item(E_CANVAS_VBOX(e_table->canvas_vbox), GNOME_CANVAS_ITEM(e_table->group));
-
-	gnome_canvas_item_set(
-		GNOME_CANVAS_ITEM(e_table->group),
-		"drawgrid", e_table->draw_grid,
-		"drawfocus", e_table->draw_focus,
-		"cursor_mode", e_table->cursor_mode,
-		"length_threshold", e_table->length_threshold,
-		"table_selection_model", e_table->selection,
-		NULL);
-
-	gtk_signal_connect (GTK_OBJECT (e_table->group), "cursor_change",
-			    GTK_SIGNAL_FUNC(group_cursor_change), e_table);
-	gtk_signal_connect (GTK_OBJECT (e_table->group), "cursor_activated",
-			    GTK_SIGNAL_FUNC(group_cursor_activated), e_table);
-	gtk_signal_connect (GTK_OBJECT (e_table->group), "double_click",
-			    GTK_SIGNAL_FUNC(group_double_click), e_table);
-	gtk_signal_connect (GTK_OBJECT (e_table->group), "right_click",
-			    GTK_SIGNAL_FUNC(group_right_click), e_table);
-	gtk_signal_connect (GTK_OBJECT (e_table->group), "click",
-			    GTK_SIGNAL_FUNC(group_click), e_table);
-	gtk_signal_connect (GTK_OBJECT (e_table->group), "key_press",
-			    GTK_SIGNAL_FUNC(group_key_press), e_table);
-
-	e_table->table_model_change_id = gtk_signal_connect (
-		GTK_OBJECT (model), "model_changed",
-		GTK_SIGNAL_FUNC (et_table_model_changed), e_table);
-
-	e_table->table_row_change_id = gtk_signal_connect (
-		GTK_OBJECT (model), "model_row_changed",
-		GTK_SIGNAL_FUNC (et_table_row_changed), e_table);
-
-	e_table->table_cell_change_id = gtk_signal_connect (
-		GTK_OBJECT (model), "model_cell_changed",
-		GTK_SIGNAL_FUNC (et_table_cell_changed), e_table);
-
-	e_table->table_row_inserted_id = gtk_signal_connect (
-		GTK_OBJECT (model), "model_row_inserted",
-		GTK_SIGNAL_FUNC (et_table_row_inserted), e_table);
-
-	e_table->table_row_deleted_id = gtk_signal_connect (
-		GTK_OBJECT (model), "model_row_deleted",
-		GTK_SIGNAL_FUNC (et_table_row_deleted), e_table);
-
+	et_build_groups(e_table);
 }
 
 static void
@@ -721,114 +778,16 @@ e_table_fill_table (ETable *e_table, ETableModel *model)
 	e_table_group_add_all (e_table->group);
 }
 
-static ETableCol *
-et_col_spec_to_col (ETable                    *e_table,
-		    ETableColumnSpecification *col_spec,
-		    ETableExtras              *ete)
-{
-	ETableCol *col = NULL;
-	ECell *cell;
-	GCompareFunc compare;
-
-	cell = e_table_extras_get_cell(ete, col_spec->cell);
-	compare = e_table_extras_get_compare(ete, col_spec->compare);
-
-	if (cell && compare) {
-		if (col_spec->pixbuf && *col_spec->pixbuf) {
-			GdkPixbuf *pixbuf;
-
-			pixbuf = e_table_extras_get_pixbuf(
-				ete, col_spec->pixbuf);
-			if (pixbuf) {
-				col = e_table_col_new_with_pixbuf (
-					col_spec->model_col, gettext (col_spec->title),
-					pixbuf, col_spec->expansion,
-					col_spec->minimum_width,
-					cell, compare, col_spec->resizable);
-			}
-		}
-		if (col == NULL && col_spec->title && *col_spec->title) {
-			col = e_table_col_new (
-				col_spec->model_col, gettext (col_spec->title),
-				col_spec->expansion, col_spec->minimum_width,
-				cell, compare, col_spec->resizable);
-		}
-	}
-	return col;
-}
-
-static ETableHeader *
-et_spec_to_full_header (ETable              *e_table,
-			ETableSpecification *spec,
-			ETableExtras        *ete)
-{
-	ETableHeader *nh;
-	int column;
-
-	g_return_val_if_fail (e_table, NULL);
-	g_return_val_if_fail (spec, NULL);
-	g_return_val_if_fail (ete, NULL);
-
-	nh = e_table_header_new ();
-
-	for (column = 0; spec->columns[column]; column++) {
-		ETableCol *col = et_col_spec_to_col (
-			e_table, spec->columns[column], ete);
-
-		if (col)
-			e_table_header_add_column (nh, col, -1);
-	}
-
-	return nh;
-}
-
-static ETableHeader *
-et_state_to_header (ETable *e_table, ETableHeader *full_header, ETableState *state)
-{
-	ETableHeader *nh;
-	const int max_cols = e_table_header_count (full_header);
-	int column;
-
-	g_return_val_if_fail (e_table, NULL);
-	g_return_val_if_fail (full_header, NULL);
-	g_return_val_if_fail (state, NULL);
-
-	nh = e_table_header_new ();
-
-	gtk_object_set(GTK_OBJECT(nh),
-		       "width_extras", e_table_header_width_extras(GTK_WIDGET(e_table)->style),
-		       NULL);
-
-	for (column = 0; column < state->col_count; column++) {
-		int col;
-		double expansion;
-		ETableCol *table_col;
-
-		col = state->columns[column];
-		expansion = state->expansions[column];
-
-		if (col >= max_cols)
-			continue;
-
-		table_col = e_table_header_get_column (full_header, col);
-
-		if (expansion >= -1)
-			table_col->expansion = expansion;
-
-		e_table_header_add_column (nh, table_col, -1);
-	}
-
-	return nh;
-}
-
 void
 e_table_set_state_object(ETable *e_table, ETableState *state)
 {
 	if (e_table->header)
 		gtk_object_unref(GTK_OBJECT(e_table->header));
-	e_table->header = et_state_to_header (e_table, e_table->full_header, state);
-	if (e_table->header)
-		gtk_object_ref(GTK_OBJECT(e_table->header));
+	e_table->header = e_table_state_to_header (GTK_WIDGET(e_table), e_table->full_header, state);
+
+	gtk_object_set (GTK_OBJECT (e_table->header),
+			"width", (double) (GTK_WIDGET(e_table->table_canvas)->allocation.width),
+			NULL);
 
 	if (e_table->sort_info) {
 		if (e_table->group_info_change_id)
@@ -1007,11 +966,12 @@ et_real_construct (ETable *e_table, ETableModel *etm, ETableExtras *ete,
 		ete = e_table_extras_new();
 
 	e_table->use_click_to_add = specification->click_to_add;
-	e_table->click_to_add_message = g_strdup (gettext (specification->click_to_add_message));
-	e_table->draw_grid = specification->draw_grid;
+	e_table->click_to_add_message = e_utf8_from_locale_string (gettext (specification->click_to_add_message));
+	e_table->horizontal_draw_grid = specification->horizontal_draw_grid;
+	e_table->vertical_draw_grid = specification->vertical_draw_grid;
 	e_table->draw_focus = specification->draw_focus;
 	e_table->cursor_mode = specification->cursor_mode;
-	e_table->full_header = et_spec_to_full_header(e_table, specification, ete);
+	e_table->full_header = e_table_spec_to_full_header(specification, ete);
 
 	gtk_object_set(GTK_OBJECT(e_table->selection),
 		       "selection_mode", specification->selection_mode,
@@ -1024,8 +984,11 @@ et_real_construct (ETable *e_table, ETableModel *etm, ETableExtras *ete,
 	gtk_widget_push_visual (gdk_rgb_get_visual ());
 	gtk_widget_push_colormap (gdk_rgb_get_cmap ());
 
-	e_table->header = et_state_to_header (e_table, e_table->full_header, state);
+	e_table->header = e_table_state_to_header (GTK_WIDGET(e_table), e_table->full_header, state);
+	e_table->horizontal_scrolling = specification->horizontal_scrolling;
+
 	e_table->sort_info = state->sort_info;
+	gtk_object_ref (GTK_OBJECT (state->sort_info));
 
 	e_table->group_info_change_id =
 		gtk_signal_connect (GTK_OBJECT (e_table->sort_info), "group_info_changed",
@@ -1360,15 +1323,15 @@ e_table_get_cursor_row (ETable *e_table)
 
 void
 e_table_selected_row_foreach     (ETable *e_table,
-				  ETableForeachFunc callback,
+				  EForeachFunc callback,
 				  gpointer closure)
 {
 	g_return_if_fail(e_table != NULL);
 	g_return_if_fail(E_IS_TABLE(e_table));
 
-	e_table_selection_model_foreach(e_table->selection,
-					callback,
-					closure);
+	e_selection_model_foreach(E_SELECTION_MODEL (e_table->selection),
+						     callback,
+						     closure);
 }
 
 gint
@@ -1377,7 +1340,7 @@ e_table_selected_count     (ETable *e_table)
 	g_return_val_if_fail(e_table != NULL, -1);
 	g_return_val_if_fail(E_IS_TABLE(e_table), -1);
 
-	return e_table_selection_model_selected_count(e_table->selection);
+	return e_selection_model_selected_count(E_SELECTION_MODEL (e_table->selection));
 }
 
 void
@@ -1386,7 +1349,7 @@ e_table_select_all (ETable *table)
 	g_return_if_fail (table != NULL);
 	g_return_if_fail (E_IS_TABLE (table));
 
-	e_table_selection_model_select_all (table->selection);
+	e_selection_model_select_all (E_SELECTION_MODEL (table->selection));
 }
 
 void
@@ -1395,7 +1358,7 @@ e_table_invert_selection (ETable *table)
 	g_return_if_fail (table != NULL);
 	g_return_if_fail (E_IS_TABLE (table));
 
-	e_table_selection_model_invert_selection (table->selection);
+	e_selection_model_invert_selection (E_SELECTION_MODEL (table->selection));
 }
 
 
@@ -1469,10 +1432,10 @@ e_table_get_next_row      (ETable *e_table,
 
 	if (e_table->sorter) {
 		int i;
-		i = e_table_sorter_model_to_sorted(e_table->sorter, model_row);
+		i = e_sorter_model_to_sorted(E_SORTER (e_table->sorter), model_row);
 		i++;
 		if (i < e_table_model_row_count(e_table->model)) {
-			return e_table_sorter_sorted_to_model(e_table->sorter, i);
+			return e_sorter_sorted_to_model(E_SORTER (e_table->sorter), i);
 		} else
 			return -1;
 	} else
@@ -1491,10 +1454,10 @@ e_table_get_prev_row      (ETable *e_table,
 
 	if (e_table->sorter) {
 		int i;
-		i = e_table_sorter_model_to_sorted(e_table->sorter, model_row);
+		i = e_sorter_model_to_sorted(E_SORTER (e_table->sorter), model_row);
 		i--;
 		if (i >= 0)
-			return e_table_sorter_sorted_to_model(e_table->sorter, i);
+			return e_sorter_sorted_to_model(E_SORTER (e_table->sorter), i);
 		else
 			return -1;
 	} else
@@ -1509,7 +1472,7 @@ e_table_model_to_view_row        (ETable *e_table,
 	g_return_val_if_fail(E_IS_TABLE(e_table), -1);
 
 	if (e_table->sorter)
-		return e_table_sorter_model_to_sorted(e_table->sorter, model_row);
+		return e_sorter_model_to_sorted(E_SORTER (e_table->sorter), model_row);
 	else
 		return model_row;
 }
@@ -1522,11 +1485,40 @@ e_table_view_to_model_row        (ETable *e_table,
 	g_return_val_if_fail(E_IS_TABLE(e_table), -1);
 
 	if (e_table->sorter)
-		return e_table_sorter_sorted_to_model(e_table->sorter, view_row);
+		return e_sorter_sorted_to_model (E_SORTER (e_table->sorter), view_row);
 	else
 		return view_row;
 }
 
+/**
+ * e_table_get_cell_at:
+ * @table: An ETable widget
+ * @x: X coordinate for the pixel
+ * @y: Y coordinate for the pixel
+ * @row_return: Pointer to return the row value
+ * @col_return: Pointer to return the column value
+ * 
+ * Return the row and column for the cell in which the pixel at (@x, @y) is
+ * contained.
+ **/
+void
+e_table_get_cell_at (ETable *table,
+		     int x, int y,
+		     int *row_return, int *col_return)
+{
+	g_return_if_fail (table != NULL);
+	g_return_if_fail (E_IS_TABLE (table));
+	g_return_if_fail (row_return != NULL);
+	g_return_if_fail (col_return != NULL);
+
+	/* FIXME it would be nice if it could handle a NULL row_return or
+	 * col_return gracefully.  */
+
+	x += GTK_LAYOUT(table->table_canvas)->hadjustment->value;
+	y += GTK_LAYOUT(table->table_canvas)->vadjustment->value;
+	e_table_group_compute_location(table->group, &x, &y, row_return, col_return);
+}
+
 struct _ETableDragSourceSite
 {
 	GdkModifierType    start_button_mask;
@@ -1705,7 +1697,7 @@ e_table_drag_source_set  (ETable               *table,
 	gtk_widget_add_events (canvas,
 			       gtk_widget_get_events (canvas) |
 			       GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
-			       GDK_BUTTON_MOTION_MASK);
+			       GDK_BUTTON_MOTION_MASK | GDK_STRUCTURE_MASK);
 
 	if (site) {
 		if (site->target_list)
@@ -1783,17 +1775,6 @@ e_table_drag_begin (ETable            *table,
 			      event);
 }
 
-static void
-e_table_compute_location (ETable *table, GtkWidget *widget,
-			  int x, int y, int *row, int *col)
-{
-	if (!(row || col))
-		return;
-	x += GTK_LAYOUT(table->table_canvas)->hadjustment->value;
-	y += GTK_LAYOUT(table->table_canvas)->vadjustment->value;
-	e_table_group_compute_location(table->group, &x, &y, row, col);
-}
-
 static void
 et_drag_begin (GtkWidget *widget,
 	       GdkDragContext *context,
@@ -1874,12 +1855,9 @@ et_drag_motion(GtkWidget *widget,
 {
 	gboolean ret_val;
 	int row, col;
-	e_table_compute_location(et,
-				 widget,
-				 x,
-				 y,
-				 &row,
-				 &col);
+
+	e_table_get_cell_at (et, x, y, &row, &col);
+
 	if (et->drop_row >= 0 && et->drop_col >= 0 &&
 	    row != et->drop_row && col != et->drop_row) {
 		gtk_signal_emit (GTK_OBJECT (et),
@@ -1914,12 +1892,9 @@ et_drag_drop(GtkWidget *widget,
 {
 	gboolean ret_val;
 	int row, col;
-	e_table_compute_location(et,
-				 widget,
-				 x,
-				 y,
-				 &row,
-				 &col);
+
+	e_table_get_cell_at (et, x, y, &row, &col);
+
 	if (et->drop_row >= 0 && et->drop_col >= 0 &&
 	    row != et->drop_row && col != et->drop_row) {
 		gtk_signal_emit (GTK_OBJECT (et),
@@ -1967,12 +1942,9 @@ et_drag_data_received(GtkWidget *widget,
 		      ETable *et)
 {
 	int row, col;
-	e_table_compute_location(et,
-				 widget,
-				 x,
-				 y,
-				 &row,
-				 &col);
+
+	e_table_get_cell_at (et, x, y, &row, &col);
+
 	gtk_signal_emit (GTK_OBJECT (et),
 			 et_signals [TABLE_DRAG_DATA_RECEIVED],
 			 row,
@@ -1997,7 +1969,7 @@ e_table_drag_source_event_cb (GtkWidget      *widget,
 	case GDK_BUTTON_PRESS:
 		if ((GDK_BUTTON1_MASK << (event->button.button - 1)) & site->start_button_mask) {
 			int row, col;
-			e_table_compute_location(table, widget, event->button.x, event->button.y, &row, &col);
+			e_table_get_cell_at (table, event->button.x, event->button.y, &row, &col);
 			if (row >= 0 && col >= 0) {
 				site->state |= (GDK_BUTTON1_MASK << (event->button.button - 1));
 				site->x = event->button.x;
@@ -2130,7 +2102,7 @@ e_table_class_init (ETableClass *class)
 				object_class->type,
 				GTK_SIGNAL_OFFSET (ETableClass, double_click),
 				gtk_marshal_NONE__INT_INT_POINTER,
-				GTK_TYPE_NONE, 3, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_POINTER);
+				GTK_TYPE_NONE, 3, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_GDK_EVENT);
 
 	et_signals [RIGHT_CLICK] =
 		gtk_signal_new ("right_click",
@@ -2138,7 +2110,7 @@ e_table_class_init (ETableClass *class)
 				object_class->type,
 				GTK_SIGNAL_OFFSET (ETableClass, right_click),
 				e_marshal_INT__INT_INT_POINTER,
-				GTK_TYPE_INT, 3, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_POINTER);
+				GTK_TYPE_INT, 3, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_GDK_EVENT);
 
 	et_signals [CLICK] =
 		gtk_signal_new ("click",
@@ -2146,7 +2118,7 @@ e_table_class_init (ETableClass *class)
 				object_class->type,
 				GTK_SIGNAL_OFFSET (ETableClass, click),
 				e_marshal_INT__INT_INT_POINTER,
-				GTK_TYPE_INT, 3, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_POINTER);
+				GTK_TYPE_INT, 3, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_GDK_EVENT);
 
 	et_signals [KEY_PRESS] =
 		gtk_signal_new ("key_press",
@@ -2154,7 +2126,7 @@ e_table_class_init (ETableClass *class)
 				object_class->type,
 				GTK_SIGNAL_OFFSET (ETableClass, key_press),
 				e_marshal_INT__INT_INT_POINTER,
-				GTK_TYPE_INT, 3, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_POINTER);
+				GTK_TYPE_INT, 3, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_GDK_EVENT);
 
 	et_signals[TABLE_DRAG_BEGIN] =
 		gtk_signal_new ("table_drag_begin",
