@@ -52,8 +52,8 @@ typedef struct {
 	int table_model_change_id;
 	int table_row_change_id;
 	int table_cell_change_id;
-	int table_row_inserted_id;
-	int table_row_deleted_id;
+	int table_rows_inserted_id;
+	int table_rows_deleted_id;
 
 	int group_info_change_id;
 
@@ -73,15 +73,20 @@ typedef struct {
 	/*
 	 * Configuration settings
 	 */
-	guint draw_grid : 1;
+	guint horizontal_draw_grid : 1;
+	guint vertical_draw_grid : 1;
 	guint draw_focus : 1;
 	guint row_selection_active : 1;
+
+	guint horizontal_scrolling : 1;
+
+	guint is_grouped : 1;
 	
 	char *click_to_add_message;
 	GnomeCanvasItem *click_to_add;
 	gboolean use_click_to_add;
 
-	ETableCursorMode cursor_mode;
+	ECursorMode cursor_mode;
 
 	int drag_get_data_row;
 	int drag_get_data_col;
@@ -207,7 +212,7 @@ void            e_table_set_cursor_row            (ETable               *e_table
 /* -1 means we don't have the cursor. */
 int             e_table_get_cursor_row            (ETable               *e_table);
 void            e_table_selected_row_foreach      (ETable               *e_table,
-						   ETableForeachFunc     callback,
+						   EForeachFunc          callback,
 						   gpointer              closure);
 gint            e_table_selected_count            (ETable               *e_table);
 EPrintable     *e_table_get_printable             (ETable               *e_table);
@@ -221,7 +226,9 @@ gint            e_table_model_to_view_row         (ETable               *e_table
 						   gint                  model_row);
 gint            e_table_view_to_model_row         (ETable               *e_table,
 						   gint                  view_row);
-
+void            e_table_get_cell_at               (ETable *table,
+						   int x, int y,
+						   int *row_return, int *col_return);
 
 /* Drag & drop stuff. */
 /* Target */
