@@ -28,13 +28,17 @@
  */
 
 #include <config.h>
+
+#include "e-icon-bar.h"
+
 #include <gtk/gtkmain.h>
+#include <gtk/gtkwindow.h>
 #include <gtk/gtksignal.h>
 #include <gdk-pixbuf/gnome-canvas-pixbuf.h>
-#include "e-icon-bar.h"
 #include "e-icon-bar-bg-item.h"
 #include <gal/e-text/e-text.h>
 #include <gal/widgets/e-canvas-utils.h>
+#include <libgnomeui/gnome-canvas-rect-ellipse.h>
 
 /* These are the offsets of the icons & text in both views. Note that the
    shadow around icons is drawn in the space between items (as is the
@@ -230,6 +234,7 @@ e_icon_bar_init (EIconBar *icon_bar)
 	icon_bar->mouse_over_item_num = -1;
 	icon_bar->editing_item_num = -1;
 	icon_bar->edit_rect_item = NULL;
+	icon_bar->enable_drags = TRUE;
 	icon_bar->in_drag = FALSE;
 	icon_bar->dragging_before_item_num = -1;
 	icon_bar->icon_x = 0;
@@ -1083,10 +1088,20 @@ e_icon_bar_item_pressed (EIconBar *icon_bar,
 
 	if (button == 1 && item_num != -1) {
 		icon_bar->mouse_over_item_num = item_num;
+#if 0
+		g_print ("Set mouse_over_item_num: %i\n", icon_bar->mouse_over_item_num);
+#endif
 		icon_bar->pressed_item_num = item_num;
 		icon_bar->pressed_x = event->button.x;
 		icon_bar->pressed_y = event->button.y;
 		gtk_widget_queue_draw (GTK_WIDGET (icon_bar));
+
+		/* We set owner_events to FALSE so we get all pointer events.
+		 */
+		gdk_pointer_grab (GTK_LAYOUT (icon_bar)->bin_window, FALSE,
+				  GDK_BUTTON_RELEASE_MASK |
+				  GDK_BUTTON1_MOTION_MASK,
+				  NULL, NULL, event->button.time);
 	} else if (button == 3) {
 		gtk_signal_emit (GTK_OBJECT (icon_bar),
 				 e_icon_bar_signals[ITEM_SELECTED],
@@ -1109,6 +1124,8 @@ e_icon_bar_item_released (EIconBar *icon_bar,
 	button = event->button.button;
 
 	if (button == 1) {
+		gdk_pointer_ungrab (event->button.time);
+
 		if (icon_bar->pressed_item_num != -1
 		    && icon_bar->pressed_item_num == icon_bar->mouse_over_item_num) {
 			gtk_signal_emit (GTK_OBJECT (icon_bar),
@@ -1129,11 +1146,23 @@ e_icon_bar_item_motion (EIconBar *icon_bar,
 {
 	gboolean need_redraw = TRUE;
 
+#if 0
+	g_print ("In e_icon_bar_item_motion\n");
+#endif
+
 	if (event && event->motion.state & GDK_BUTTON1_MASK
 	    && icon_bar->pressed_item_num != -1) {
-		if (abs (event->motion.x - icon_bar->pressed_x) > E_ICON_BAR_DRAG_START_OFFSET
-		    || abs (event->motion.y - icon_bar->pressed_y) > E_ICON_BAR_DRAG_START_OFFSET) {
+		if (icon_bar->enable_drags
+		    && (abs (event->motion.x - icon_bar->pressed_x) > E_ICON_BAR_DRAG_START_OFFSET
+			|| abs (event->motion.y - icon_bar->pressed_y) > E_ICON_BAR_DRAG_START_OFFSET)) {
 			icon_bar->dragged_item_num = icon_bar->pressed_item_num;
+
+			/* We ungrab the pointer and emit the item-dragged
+			   signal. Note that a drag must be started here or
+			   the icon bar may act strange as it no longer has
+			   a pointer grab. */
+			gdk_pointer_ungrab (event->motion.time);
+
 			gtk_signal_emit (GTK_OBJECT (icon_bar),
 					 e_icon_bar_signals[ITEM_DRAGGED],
 					 event, icon_bar->dragged_item_num);
@@ -1142,8 +1171,6 @@ e_icon_bar_item_motion (EIconBar *icon_bar,
 			icon_bar->pressed_item_num = -1;
 			gtk_widget_queue_draw (GTK_WIDGET (icon_bar));
 		}
-
-		return;
 	}
 
 	if (icon_bar->mouse_over_item_num == item_num)
@@ -1161,6 +1188,9 @@ e_icon_bar_item_motion (EIconBar *icon_bar,
 	}
 
 	icon_bar->mouse_over_item_num = item_num;
+#if 0
+	g_print ("Set mouse_over_item_num: %i\n", icon_bar->mouse_over_item_num);
+#endif
 
 	if (need_redraw)
 		gtk_widget_queue_draw (GTK_WIDGET (icon_bar));
@@ -1352,7 +1382,7 @@ e_icon_bar_start_editing_item (EIconBar *icon_bar,
 
 	item = &g_array_index (icon_bar->items,
 			       EIconBarItem, item_num);
-	e_canvas_item_grab_focus (item->text);
+	e_canvas_item_grab_focus (item->text, TRUE);
 }
 
 
@@ -1579,6 +1609,9 @@ e_icon_bar_drag_leave (GtkWidget      *widget,
 
 	if (icon_bar->mouse_over_item_num != -1) {
 		icon_bar->mouse_over_item_num = -1;
+#if 0
+		g_print ("Set mouse_over_item_num: %i\n", icon_bar->mouse_over_item_num);
+#endif
 		gtk_widget_queue_draw (GTK_WIDGET (icon_bar));
 	}
 }
@@ -1643,3 +1676,15 @@ e_icon_bar_vadjustment_value_changed (GtkAdjustment *adjustment,
 	e_icon_bar_update_highlight (icon_bar);
 }
 
+
+
+/* Set whether items can be dragged, for drag-and-drop. */
+void
+e_icon_bar_set_enable_drags	(EIconBar	 *icon_bar,
+				 gboolean	  enable_drags)
+{
+	g_return_if_fail (E_IS_ICON_BAR (icon_bar));
+
+	icon_bar->enable_drags = enable_drags;
+}
+
