@@ -1,8 +1,11 @@
 #ifndef GNUMERIC_COLOR_PALETTE_H
 #define GNUMERIC_COLOR_PALETTE_H
 
-#include <libgnome/gnome-defs.h>
+#include <gtk/gtktooltips.h>
+#include <gtk/gtkvbox.h>
 #include <gtk/gtkwidget.h>
+#include <libgnome/gnome-defs.h>
+#include <libgnomeui/gnome-color-picker.h>
 #include <gal/widgets/color-group.h>
 
 BEGIN_GNOME_DECLS
@@ -58,6 +61,6 @@ GtkWidget       *color_palette_get_color_picker (ColorPalette *P);
 
 END_GNOME_DECLS
 
-#endif GNUMERIC_PALETTE_H
+#endif /* GNUMERIC_PALETTE_H */
 
 
