@@ -20,7 +20,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <gnome.h>
+#include <gtk/gtksignal.h>
 #include "e-canvas.h"
 
 static void e_canvas_init           (ECanvas         *card);
@@ -31,6 +31,10 @@ static void e_canvas_unrealize      (GtkWidget        *widget);
 static gint e_canvas_key            (GtkWidget        *widget,
 				     GdkEventKey      *event);
 
+static gint e_canvas_visibility     (GtkWidget        *widget,
+				     GdkEventVisibility *event,
+				     ECanvas          *canvas);
+
 static gint e_canvas_focus_in       (GtkWidget        *widget,
 				     GdkEventFocus    *event);
 static gint e_canvas_focus_out      (GtkWidget        *widget,
@@ -114,6 +118,7 @@ e_canvas_init (ECanvas *canvas)
 	canvas->cursor = NULL;
 	canvas->ic = NULL;
 	canvas->ic_attr = NULL;
+	canvas->tooltip_window = NULL;
 }
 
 static void
@@ -124,6 +129,16 @@ e_canvas_destroy (GtkObject *object)
 	if (canvas->idle_id)
 		g_source_remove(canvas->idle_id);
 
+	if (canvas->toplevel) {
+		if (canvas->visibility_notify_id)
+			gtk_signal_disconnect (GTK_OBJECT(canvas->toplevel),
+					       canvas->visibility_notify_id);
+		canvas->visibility_notify_id = 0;
+
+		gtk_object_unref(GTK_OBJECT(canvas->toplevel));
+		canvas->toplevel = NULL;
+	}
+
 	if ((GTK_OBJECT_CLASS (parent_class))->destroy)
 		(*(GTK_OBJECT_CLASS (parent_class))->destroy) (object);
 }
@@ -271,27 +286,43 @@ static gint
 e_canvas_key (GtkWidget *widget, GdkEventKey *event)
 {
 	GnomeCanvas *canvas;
+	GdkEvent full_event;
 
 	g_return_val_if_fail (widget != NULL, FALSE);
 	g_return_val_if_fail (GNOME_IS_CANVAS (widget), FALSE);
 	g_return_val_if_fail (event != NULL, FALSE);
 
 	canvas = GNOME_CANVAS (widget);
+	
+	full_event.key = *event;
 
-	return emit_event (canvas, (GdkEvent *) event);
+	return emit_event (canvas, &full_event);
+}
+
+/* Key event handler for the canvas */
+static gint
+e_canvas_visibility (GtkWidget *widget, GdkEventVisibility *event, ECanvas *canvas)
+{
+	if (! canvas->visibility_first) {
+		e_canvas_hide_tooltip(canvas);
+	}
+	canvas->visibility_first = FALSE;
+
+	return FALSE;
 }
 
 
 /**
  * e_canvas_item_grab_focus:
  * @item: A canvas item.
+ * @widget_too: Whether or not to grab the widget-level focus too
  *
- * Makes the specified item take the keyboard focus, so all keyboard events will
- * be sent to it.  If the canvas widget itself did not have the focus, it grabs
- * it as well.
+ * Makes the specified item take the keyboard focus, so all keyboard
+ * events will be sent to it. If the canvas widget itself did not have
+ * the focus and @widget_too is %TRUE, it grabs that focus as well.
  **/
 void
-e_canvas_item_grab_focus (GnomeCanvasItem *item)
+e_canvas_item_grab_focus (GnomeCanvasItem *item, gboolean widget_too)
 {
 	GnomeCanvasItem *focused_item;
 	GdkEvent ev;
@@ -313,7 +344,7 @@ e_canvas_item_grab_focus (GnomeCanvasItem *item)
 
 	item->canvas->focused_item = item;
 
-	if (!GTK_WIDGET_HAS_FOCUS (GTK_WIDGET(item->canvas))) {
+	if (widget_too && !GTK_WIDGET_HAS_FOCUS (GTK_WIDGET(item->canvas))) {
 		gtk_widget_grab_focus (GTK_WIDGET (item->canvas));
 	}
 
@@ -333,6 +364,7 @@ e_canvas_focus_in (GtkWidget *widget, GdkEventFocus *event)
 {
 	GnomeCanvas *canvas;
 	ECanvas *ecanvas;
+	GdkEvent full_event;
 
 	canvas = GNOME_CANVAS (widget);
 	ecanvas = E_CANVAS (widget);
@@ -342,10 +374,12 @@ e_canvas_focus_in (GtkWidget *widget, GdkEventFocus *event)
 	if (ecanvas->ic)
 		gdk_im_begin (ecanvas->ic, canvas->layout.bin_window);
 
-	if (canvas->focused_item)
-		return emit_event (canvas, (GdkEvent *) event);
-	else
+	if (canvas->focused_item) {
+		full_event.focus_change = *event;
+		return emit_event (canvas, &full_event);
+	} else {
 		return FALSE;
+	}
 }
 
 /* Focus out handler for the canvas */
@@ -354,6 +388,7 @@ e_canvas_focus_out (GtkWidget *widget, GdkEventFocus *event)
 {
 	GnomeCanvas *canvas;
 	ECanvas *ecanvas;
+	GdkEvent full_event;
 
 	canvas = GNOME_CANVAS (widget);
 	ecanvas = E_CANVAS (widget);
@@ -363,10 +398,12 @@ e_canvas_focus_out (GtkWidget *widget, GdkEventFocus *event)
 	if (ecanvas->ic)
 		gdk_im_end ();
 
-	if (canvas->focused_item)
-		return emit_event (canvas, (GdkEvent *) event);
-	else
+	if (canvas->focused_item) {
+		full_event.focus_change = *event;
+		return emit_event (canvas, &full_event);
+	} else {
 		return FALSE;
+	}
 }
 
 static void
@@ -693,3 +730,31 @@ e_canvas_item_remove_selection (GnomeCanvasItem *item, gpointer id)
 		}
 	}
 }
+
+void e_canvas_popup_tooltip (ECanvas *canvas, GtkWidget *widget, int x, int y)
+{
+	if (canvas->tooltip_window && canvas->tooltip_window != widget) {
+		e_canvas_hide_tooltip(canvas);
+	}
+	canvas->tooltip_window = widget;
+	canvas->visibility_first = TRUE;
+	if (canvas->toplevel == NULL) {
+		canvas->toplevel = gtk_widget_get_toplevel (GTK_WIDGET(canvas));
+		if (canvas->toplevel) {
+			gtk_widget_add_events(canvas->toplevel, GDK_VISIBILITY_NOTIFY_MASK);
+			gtk_object_ref(GTK_OBJECT(canvas->toplevel));
+			canvas->visibility_notify_id =
+				gtk_signal_connect (GTK_OBJECT (canvas->toplevel), "visibility_notify_event",
+						    GTK_SIGNAL_FUNC (e_canvas_visibility), canvas);
+		}
+	}
+	gtk_widget_popup (widget, x, y);
+}
+
+void e_canvas_hide_tooltip  (ECanvas *canvas)
+{
+	if (canvas->tooltip_window) {
+		gtk_widget_destroy (canvas->tooltip_window);
+		canvas->tooltip_window = NULL;
+	}
+}
