@@ -12,7 +12,7 @@
  *
  */
 
-#include "config.h"
+#include <config.h>
 #include <string.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
