@@ -6,6 +6,8 @@
 
 #include <libgnomeui/gnome-messagebox.h>
 
+BEGIN_GNOME_DECLS
+
 void  e_popup_menu                   (GtkMenu *menu, GdkEvent *event);
 void  e_auto_kill_popup_menu_on_hide (GtkMenu *menu);
 void  e_notice                       (GtkWindow *window, const char *type, const char *format, ...);
@@ -17,4 +19,6 @@ void e_container_focus_nth_entry     (GtkContainer *container,
 gint e_container_change_tab_order    (GtkContainer *container,
 				      GList        *widgets);
 
+END_GNOME_DECLS
+
 #endif /* GAL_GUI_UTILS_H */
