@@ -2,7 +2,11 @@
 #ifndef E_POPUP_MENU_H
 #define E_POPUP_MENU_H
 
+#include <gtk/gtkmenu.h>
 #include <gtk/gtkwidget.h>
+#include <libgnome/gnome-defs.h>
+
+BEGIN_GNOME_DECLS
 
 typedef struct _EPopupMenu EPopupMenu;
 
@@ -25,4 +29,6 @@ void     e_popup_menu_run     (EPopupMenu     *menu_list,
 			       guint32         hide_mask,
 			       void           *closure);
 
+END_GNOME_DECLS
+
 #endif /* E_POPUP_MENU_H */
