@@ -10,12 +10,15 @@
  *
  */
 #include <config.h>
-#include "widget-color-combo.h"
 
-#include <gnome.h>
 #include <gtk/gtkentry.h>
+#include <gtk/gtksignal.h>
 #include <libgnomeui/gnome-canvas.h>
+#include <libgnomeui/gnome-canvas-image.h>
+#include <libgnomeui/gnome-canvas-rect-ellipse.h>
+#include <libgnomeui/gnome-preferences.h>
 #include "e-colors.h"
+#include "widget-color-combo.h"
 
 enum {
 	CHANGED,
@@ -28,8 +31,8 @@ static GtkObjectClass *color_combo_parent_class;
 
 #define make_color(CC,COL) (((COL) != NULL) ? (COL) : ((CC) ? ((CC)->default_color) : NULL))
 
-static void
-set_color (ColorCombo *cc, GdkColor *color)
+void
+color_combo_set_color (ColorCombo *cc, GdkColor *color)
 {
 	GdkColor *new_color;
 	GdkColor *outline_color;
@@ -108,7 +111,7 @@ emit_change (ColorCombo *cc, GdkColor *color)
 static void
 cb_color_change (ColorPalette *P, GdkColor *color, gboolean custom, ColorCombo *cc)
 {
-	set_color (cc, color);
+	color_combo_set_color (cc, color);
 	emit_change (cc, color);
 }
 
@@ -237,7 +240,7 @@ color_combo_construct (ColorCombo *cc, char **icon,
 		gtk_combo_box_set_arrow_relief (GTK_COMBO_BOX (cc), GTK_RELIEF_NONE);
 
 	/* Start with the default color */
-	set_color (cc, NULL);
+	color_combo_set_color (cc, NULL);
 }
 
 /*
