@@ -1,8 +1,8 @@
 #ifndef GNUMERIC_WIDGET_COLOR_COMBO_H
 #define GNUMERIC_WIDGET_COLOR_COMBO_H
 
-#include <libgnome/gnome-defs.h>
 #include <gtk/gtkwidget.h>
+#include <libgnome/gnome-defs.h>
 #include <gal/widgets/gtk-combo-box.h>
 #include <gal/widgets/color-palette.h>
 
@@ -35,11 +35,14 @@ typedef struct {
 #define IS_COLOR_COMBO(obj)  (GTK_CHECK_TYPE((obj), COLOR_COMBO_TYPE))
 
 
-GtkType    color_combo_get_type (void);
-
-GtkWidget *color_combo_new      (char **icon, const char *no_color_label,
-				 GdkColor *default_color, gchar *group_name);
+GtkType    color_combo_get_type   (void);
+GtkWidget *color_combo_new        (char       **icon,
+				   const char  *no_color_label,
+				   GdkColor    *default_color,
+				   gchar       *group_name);
+void       color_combo_set_color  (ColorCombo  *cc,
+				   GdkColor    *color);
 
 END_GNOME_DECLS
 
-#endif GNUMERIC_WIDGET_COLOR_COMBO_H
+#endif /* GNUMERIC_WIDGET_COLOR_COMBO_H */
