@@ -138,6 +138,10 @@ main (int argc, char *argv[])
 	e_shortcut_bar_set_icon_callback (E_SHORTCUT_BAR (shortcut_bar),
 					  icon_callback,
 					  NULL);
+#if 0
+	e_shortcut_bar_set_enable_drags (E_SHORTCUT_BAR (shortcut_bar),
+					 FALSE);
+#endif
 
 	gtk_signal_connect (GTK_OBJECT (shortcut_model), "item_added",
 			    GTK_SIGNAL_FUNC (on_item_added), NULL);
@@ -221,7 +225,8 @@ icon_callback (EShortcutBar *shortcut_bar,
 	for (i = 0; i < NUM_SHORTCUT_TYPES; i++) {
 		if (!strncmp (url, shortcut_types[i],
 			      strlen (shortcut_types[i]))) {
-			gdk_pixbuf_ref (icon_pixbufs[i]);
+			if (icon_pixbufs[i])
+				gdk_pixbuf_ref (icon_pixbufs[i]);
 			return icon_pixbufs[i];
 		}
 	}
